from cpm.infrastructure import filesystem
from cpm.domain.constants import PROJECT_DESCRIPTOR_FILE


class TemplatePackager(object):
    def pack(self, project_descriptor, build_directory):
        if filesystem.directory_exists(build_directory):
            raise FailedToPackageTemplate(cause='build directory exists')

        filesystem.create_directory(build_directory)
        filesystem.copy_file(PROJECT_DESCRIPTOR_FILE, f'{build_directory}/project.yaml')
        self.__copy_packages(build_directory, project_descriptor)
        self.__copy_mains(build_directory, project_descriptor)
        self.__copy_dockerfiles(build_directory, project_descriptor)
        filesystem.zip(build_directory, f'{project_descriptor.name}')
        filesystem.remove_directory(build_directory)

        return f'{project_descriptor.name}.zip'

    def __copy_dockerfiles(self, build_directory, project_descriptor):
        for target, description in project_descriptor.targets.items():
            if description.dockerfile and filesystem.file_exists(description.dockerfile):
                filesystem.copy_file(description.dockerfile, f'{build_directory}/')

    def __copy_mains(self, build_directory, project_descriptor):
        for target, description in project_descriptor.targets.items():
            relative_path_from_root = filesystem.path_to(description.main)
            if relative_path_from_root:
                filesystem.create_directory(f'{build_directory}/{relative_path_from_root}')
            filesystem.copy_file(description.main, f'{build_directory}/{relative_path_from_root}')

    def __copy_packages(self, build_directory, project_descriptor):
        for package in project_descriptor.build_packages():
            filesystem.copy_directory(package.path, f'{build_directory}/{package.path}')
        for package in project_descriptor.test.packages:
            filesystem.copy_directory(package.path, f'{build_directory}/{package.path}')


class FailedToPackageTemplate(RuntimeError):
    def __init__(self, cause='packaging failure'):
        self.cause = cause



import unittest
import os
import shutil

from cpm.domain.project import project_descriptor_parser
from cpm.infrastructure import filesystem


class TestSchemaValidation(unittest.TestCase):
    PROJECT_NAME = 'test_project'
    TEST_DIRECTORY = f'{os.path.dirname(os.path.abspath(__file__))}'
    PROJECT_DIRECTORY = f'{TEST_DIRECTORY}/{PROJECT_NAME}'

    def setUp(self):
        self.cwd = os.getcwd()
        shutil.rmtree(self.PROJECT_DIRECTORY, ignore_errors=True)

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(self.PROJECT_DIRECTORY, ignore_errors=True)

    def test_project_name_is_required(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """pepe: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'name is required'

    def test_project_name_must_be_a_string(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:1:1: name must be a string'

    def test_project_version_must_be_a_string(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:2:1: version must be a string'

    def disabled_test_project_version_must_be_a_valid_semver_value(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "pepin"
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:2:1: version must be a valid semver string'

    def test_build_compilation_plan_must_be_a_mapping(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:3:1: build must be a mapping'

    def test_bits_in_compilation_plan_must_be_a_mapping(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  bits: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:3: bits must be a mapping'

    def test_bits_in_compilation_plan_can_be_empty(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  bits:
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

    def test_bits_configuration_in_compilation_plan(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  bits:
    bit_name: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:5:5: bit_name must be a string or a mapping'

    def test_packages_in_compilation_plan_must_be_a_mapping(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  packages: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:3: packages must be a mapping'

    def test_package_description_in_compilation_plan_must_be_a_mapping(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  packages:
    package_name: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:5:5: package_name must be a mapping'

    def test_cflags_in_package_description_in_compilation_plan_must_be_a_sequence(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  packages:
    package_name:
      cflags: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:6:7: cflags must be a sequence of string'

    def test_cflags_in_package_description_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  packages:
    package_name:
      cflags: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:6:16: cflags must be a sequence of string'

    def test_cflags_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  cflags: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:12: cflags must be a sequence of string'

    def test_cppflags_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  cppflags: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:14: cppflags must be a sequence of string'

    def test_ldflags_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  ldflags: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:13: ldflags must be a sequence of string'

    def test_libraries_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  libraries: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:15: libraries must be a sequence of string'

    def test_includes_in_compilation_plan_must_be_a_sequence_of_strings(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
build:
  includes: [123, '123']
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:4:14: includes must be a sequence of string'

    def test_test_compilation_plan_must_be_a_mapping(self):
        os.mkdir(self.PROJECT_DIRECTORY)
        descriptor_contents = """name: test_project
version: "1.0.0"
test: 123
"""
        filesystem.create_file(f'{self.PROJECT_DIRECTORY}/project.yaml', descriptor_contents)
        with self.assertRaises(project_descriptor_parser.ParseError) as context:
            project_descriptor_parser.parse_from(self.PROJECT_DIRECTORY)

        assert str(context.exception.message) == 'project.yaml:3:1: test must be a mapping'



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.creation_service import CreationService
from cpm.domain.creation_service import CreationOptions
from cpm.domain.project.project_loader import ProjectLoader
from cpm.infrastructure.cpm_hub_connector_v1 import CpmHubConnectorV1, TemplateNotFound


def new_project(creation_service, options=CreationOptions()):
    try:
        if creation_service.exists(options.directory):
            return Result(FAIL, f'error: directory {options.directory} already exists')
        creation_service.create(options)
    except TemplateNotFound:
        return Result(FAIL, f'error: template {options.template_name}:{options.template_version} not found')
    return Result(OK, f'Created project {options.project_name}')


def execute(argv):
    create_parser = argument_parser()
    args = create_parser.parse_args(argv)

    project_loader = ProjectLoader()
    cpm_hub_connector = CpmHubConnectorV1(repository_url=args.repository_url)
    service = CreationService(project_loader, cpm_hub_connector=cpm_hub_connector)
    template_name, template_version = __template_to_use(args.template)

    options = CreationOptions(
        project_name=args.project_name,
        directory=args.project_name,
        init_from_existing_sources=False,
        init_from_template=True if args.template else False,
        template_name=template_name,
        template_version=template_version
    )
    result = new_project(service, options)

    return result


def argument_parser():
    create_parser = ArgumentParser(prog='cpm create', description='Create a new cpm project')
    create_parser.add_argument('project_name',
                               help='name of the project to create')
    create_parser.add_argument('-s', '--repository-url',
                               required=False,
                               action='store',
                               arg_format='<repository_url>',
                               help='URL of the cpm repository, used when creating project from template (default https://repo.cpmbits.com:8000)',
                               default='https://repo.cpmbits.com:8000')
    create_parser.add_argument('-t', '--template',
                               arg_format='<template>:<version>',
                               required=False,
                               help='template name and version to use when creating project from template',
                               action='store')
    return create_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'create a new cpm project'


def __template_to_use(bit_argument):
    parts = f'{bit_argument}:latest'.split(':')
    return parts[0], parts[1]



from pathlib import Path

from ruamel.yaml import YAML, RoundTripConstructor
from ruamel.yaml.nodes import SequenceNode, MappingNode


class CpmConstructor(RoundTripConstructor):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.add_constructor(u'!include', self.include)
        self.files = []
        self.child_parsers = []

    def include(self, loader, node):
        y = loader.loader
        yaml = YamlParser(typ=y.typ, pure=y.pure)
        self.child_parsers.append(yaml)
        yaml.composer.anchors = loader.composer.anchors
        path = Path(node.value)
        self.files.append(path)
        return yaml.load_from(path)

    def construct_object(self, node, deep=False):
        data = super().construct_object(node, deep)
        if isinstance(node, MappingNode) or isinstance(node, SequenceNode):
            setattr(data, 'parsing_file', self.parsing_file)
        return data

    def set_parsing_file(self, parsing_file):
        setattr(self, 'parsing_file', parsing_file)

    def included_files(self):
        return self.files + [f for p in self.child_parsers for f in p.included_files()]


class YamlParser(YAML):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.Constructor = CpmConstructor

    def load_from(self, stream):
        setattr(self, 'parsing_file', stream)
        self.constructor.set_parsing_file(stream.name)
        return self.load(stream)

    def included_files(self):
        return self.constructor.included_files()


def load(path):
    yaml = YamlParser()
    return yaml.load_from(Path(path))


def dump(data, path):
    yaml = YamlParser(typ='rt')
    yaml.dump(data, Path(path))



import unittest
import mock

from cpm.api.install import install_bit
from cpm.api.install import install_project_bits
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.infrastructure.cpm_hub_connector_v1 import BitNotFound
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.infrastructure.http_client import HttpConnectionError


class TestApiInstall(unittest.TestCase):
    def test_bit_install_fails_when_current_directory_is_not_a_cpm_project(self):
        install_service = mock.MagicMock()
        install_service.install.side_effect = ProjectDescriptorNotFound

        result = install_bit(install_service, 'cest')

        assert result.status_code == FAIL
        install_service.install.assert_called_once_with('cest', 'latest')

    def test_build_fails_when_descriptor_contains_errors(self):
        install_service = mock.MagicMock()
        install_service.install.side_effect = ParseError('project.yaml', 1, 1, 'error')

        result = install_bit(install_service, 'cest')

        assert result.status_code == FAIL
        install_service.install.assert_called_once_with('cest', 'latest')

    def test_bit_install_fails_when_http_connection_fails(self):
        install_service = mock.MagicMock()
        install_service.install.side_effect = HttpConnectionError

        result = install_bit(install_service, 'cest')

        assert result.status_code == FAIL
        install_service.install.assert_called_once_with('cest', 'latest')

    def test_bit_install_fails_when_bit_is_not_found(self):
        install_service = mock.MagicMock()
        install_service.install.side_effect = BitNotFound

        result = install_bit(install_service, 'cest')

        assert result.status_code == FAIL
        install_service.install.assert_called_once_with('cest', 'latest')

    def test_bit_install(self):
        install_service = mock.MagicMock()

        result = install_bit(install_service, 'cest')

        assert result.status_code == OK
        install_service.install.assert_called_once_with('cest', 'latest')

    def test_bit_install_of_specific_version(self):
        install_service = mock.MagicMock()

        result = install_bit(install_service, 'cest', '1.1')

        assert result.status_code == OK
        install_service.install.assert_called_once_with('cest', '1.1')

    def test_bit_install_of_all_bits_in_project(self):
        install_service = mock.MagicMock()

        result = install_project_bits(install_service)

        assert result.status_code == OK
        install_service.install_all.assert_called_once()

    def test_bit_install_of_all_bits_in_project_fails_when_current_directory_is_not_a_cpm_project(self):
        install_service = mock.MagicMock()
        install_service.install_all.side_effect = ProjectDescriptorNotFound

        result = install_project_bits(install_service)

        assert result.status_code == FAIL

    def test_bit_install_of_all_bits_in_project_fails_when_http_connection_fails(self):
        install_service = mock.MagicMock()
        install_service.install_all.side_effect = HttpConnectionError

        result = install_project_bits(install_service)

        assert result.status_code == FAIL

    def test_bit_install_of_all_bits_in_project_fails_when_one_bit_does_not_exist(self):
        install_service = mock.MagicMock()
        install_service.install_all.side_effect = BitNotFound

        result = install_project_bits(install_service)

        assert result.status_code == FAIL





import argparse

from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.creation_service import CreationService
from cpm.domain.creation_service import CreationOptions
from cpm.domain.project.project_loader import ProjectLoader


def init_project(creation_service, options=CreationOptions(init_from_existing_sources=True)):
    if creation_service.exists(options.directory):
        directory_print = 'current directory' if options.directory == '.' else f'directory {options.directory}'
        return Result(FAIL, f'error: {directory_print} already contains a cpm project')

    creation_service.create(options)
    return Result(OK, f'Created project {options.project_name}')


def execute(argv):
    args = argument_parser().parse_args(argv)

    project_loader = ProjectLoader()
    service = CreationService(project_loader)

    options = CreationOptions(
        init_from_existing_sources=True,
        generate_sample_code=False,
        directory='.',
        project_name=args.project_name
    )
    result = init_project(service, options)

    return result


def argument_parser():
    create_parser = argparse.ArgumentParser(prog='cpm init', description=description())
    create_parser.add_argument('project_name',
                               help='name of the project to create')
    return create_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'initialize the current directory as a cpm project'



import unittest
import mock

from cpm.domain.project.project_descriptor import ProjectDescriptor
from cpm.domain.publish_service import PublishService
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound
from cpm.domain.project_packager import PackagingFailure
from cpm.infrastructure.cpm_hub_connector_v1 import AuthenticationFailure


class TestPublishService(unittest.TestCase):
    @mock.patch('cpm.domain.publish_service.project_descriptor_parser')
    def test_publish_service_fails_when_loader_fails_to_load_project(self, project_descriptor_parser):
        project_descriptor_parser.parse_from.side_effect = ProjectDescriptorNotFound
        project_packager = mock.MagicMock()
        cpm_hub_connector = mock.MagicMock()
        service = PublishService(cpm_hub_connector, project_packager=project_packager)

        self.assertRaises(ProjectDescriptorNotFound, service.publish)

    @mock.patch('cpm.domain.publish_service.project_descriptor_parser')
    def test_publish_service_fails_when_packing_project_fails(self, project_descriptor_parser):
        project_descriptor = ProjectDescriptor('cpm-hub')
        project_descriptor_parser.parse_from.return_value = project_descriptor
        project_packager = mock.MagicMock()
        project_packager.pack.side_effect = PackagingFailure
        cpm_hub_connector = mock.MagicMock()
        service = PublishService(cpm_hub_connector, project_packager=project_packager)

        self.assertRaises(PackagingFailure, service.publish)

        project_packager.pack.assert_called_once_with(project_descriptor, 'dist')

    @mock.patch('cpm.domain.publish_service.project_descriptor_parser')
    def test_publish_service_fails_when_uploading_bit_fails(self, project_descriptor_parser):
        project_descriptor = ProjectDescriptor('cpm-hub')
        project_descriptor_parser.parse_from.return_value = project_descriptor
        project_packager = mock.MagicMock()
        project_packager.pack.return_value = 'packaged_file.zip'
        cpm_hub_connector = mock.MagicMock()
        cpm_hub_connector.publish_bit.side_effect = AuthenticationFailure
        service = PublishService(cpm_hub_connector, project_packager=project_packager)

        self.assertRaises(AuthenticationFailure, service.publish)

        cpm_hub_connector.publish_bit.assert_called_once_with(project_descriptor, 'packaged_file.zip')

    @mock.patch('cpm.domain.publish_service.project_descriptor_parser')
    def test_publish_service_loads_project_then_packages_it_and_uploads_it(self, project_descriptor_parser):
        project_descriptor = ProjectDescriptor('cpm-hub')
        project_descriptor_parser.parse_from.return_value = project_descriptor
        project_packager = mock.MagicMock()
        project_packager.pack.return_value = 'packaged_file.zip'
        cpm_hub_connector = mock.MagicMock()
        service = PublishService(cpm_hub_connector, project_packager=project_packager)

        service.publish()

        project_packager.pack.assert_called_once_with(project_descriptor, 'dist')
        cpm_hub_connector.publish_bit.assert_called_once_with(project_descriptor, 'packaged_file.zip')



#!/usr/bin/env python3.7
import importlib
import sys
import pkgutil
import pkg_resources
import datetime
import argparse

from cpm.argument_parser import ArgumentParser
from cpm.api.result import FAIL
import cpm.api


def handle_keyboard_interrupt(func):
    def wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except:
            print(f'\rcpm: cancelled')
            sys.exit(FAIL)
        return result
    return wrapper


@handle_keyboard_interrupt
def main():
    commands = available_commands()
    top_level_parser = argument_parser(commands)

    args = top_level_parser.parse_args(sys.argv[1:])

    if not args.command:
        print_help(top_level_parser, commands)
        sys.exit(0)

    command_to_execute = args.command[0]
    command_arguments = args.command[1:]

    if command_to_execute == 'help':
        if len(command_arguments) == 0:
            print_help(top_level_parser, commands)
        else:
            commands[args.command[1]].print_help()
            print()
        sys.exit(0)

    if command_to_execute not in commands:
        print(f'cpm: error: unknown command \'{command_to_execute}\', see \'cpm help\' for a list of available commands')
        sys.exit(1)

    start_time = datetime.datetime.now()

    result = commands[command_to_execute].execute(command_arguments)

    elapsed_time = datetime.datetime.now() - start_time
    finish(result, elapsed_time)


def available_commands():
    commands = {}
    for api_action in api_commands():
        module = importlib.import_module(api_action.name)
        if hasattr(module, 'execute'):
            commands[module_name(api_action)] = module
    return commands


def argument_parser(commands):
    top_level_parser = ArgumentParser(description='cpm: a modern project management tool for C/C++ projects')
    top_level_parser.add_argument('-v', '--version',
                                  action='version',
                                  help='show version and exit',
                                  version=f'cpm version {pkg_resources.require("cpm-cli")[0].version}')
    top_level_parser.add_argument('command',
                                  choices=list(commands.keys()) + ['help'],
                                  nargs=argparse.REMAINDER)
    return top_level_parser


def print_help(top_level_parser, commands):
    help_text = 'Usage: cpm [-h | --help] [-v | --version] <command> [args]'

    help_text += '\n\nOptional Arguments:'
    for option in top_level_parser.options:
        if 'command' in option['flags']:
            continue
        help_text += f'\n\t{", ".join(option["flags"]):<20}{option["help"]}'

    help_text += '\n\nAvailable commands:'
    for command in sorted(commands.keys()):
        help_text += f'\n\t{command:<20}{commands[command].description()}'

    help_text += '\n\nYou can get more detailed help about any specific command by running:\ncpm help <command>'
    help_text += '\n\n'

    print(help_text)


def module_name(api_command):
    return api_command.name.split('.')[-1]


def api_commands():
    return list(pkgutil.iter_modules(cpm.api.__path__, cpm.api.__name__+'.'))


def finish(result, elapsed_time):
    print(f'cpm: {result.message}  (took {__format(elapsed_time)})')
    sys.exit(result.status_code)


def __format(elapsed_time):
    if elapsed_time.seconds >= 60:
        return '%dm %d.%ds' % (elapsed_time.seconds/60, elapsed_time.seconds % 60, elapsed_time.microseconds/1000)
    else:
        return '%d.%ds' % (elapsed_time.seconds, elapsed_time.microseconds/1000)


if __name__ == '__main__':
    main()



import os
import json
from flask import Flask, request


app = Flask(__name__)
BASE_DIRECTORY = f'{os.path.dirname(os.path.abspath(__file__))}'


@app.route('/')
def hello():
    return 'hello'


@app.route('/bits/<name>/<version>', methods=['POST', 'GET'])
def bit_resource(name, version):
    if request.method == 'GET':
        with open(f'{BASE_DIRECTORY}/samples/{name}_{version}.json') as payload_file:
            payload = payload_file.read()
        return payload
    if request.method == 'POST':
        with open(f'{name}_{version}.json') as payload_file:
            payload_file.write(request.data)


@app.route('/templates/<name>/<version>', methods=['GET'])
def get_template(name, version):
    if request.method == 'GET':
        with open(f'{BASE_DIRECTORY}/templates/{name}_{version}.json') as payload_file:
            payload = payload_file.read()
        return payload


@app.route('/templates', methods=['POST'])
def publish_template():
    body = json.loads(request.data)
    with open(f'{BASE_DIRECTORY}/templates/{body["template_name"]}_{body["version"]}.json', 'wb') as payload_file:
        payload_file.write(request.data)
    return '', 200


if __name__ == '__main__':
    app.run(debug=True, host='0.0.0.0', port=8000)



import json
import base64
from getpass import getpass
from http import HTTPStatus

from cpm.domain.template_download import TemplateDownload
from cpm.infrastructure import http_client
from cpm.infrastructure import filesystem
from cpm.domain.bit_download import BitDownload


class CpmHubConnectorV1(object):
    def __init__(self, repository_url='https://repo.cpmbits.com:8000', dry_run=False):
        self.dry_run = dry_run
        self.repository_url = repository_url

    def publish_bit(self, project_descriptor, file_name):
        payload = base64.b64encode(filesystem.read_file(file_name, 'rb')).decode('utf-8')
        username = input('username: ')
        password = getpass(prompt='password: ', stream=None)
        body = {
            'bit_name': project_descriptor.name,
            'version': project_descriptor.version,
            'payload': payload,
            'username': username,
            'password': password,
        }

        if self.dry_run:
            print(f'cpm: publishing to {self.repository_url}')
            print(json.dumps(body))
            return

        response = http_client.post(f'{self.repository_url}/bits', data=json.dumps(body))
        if response.status_code == HTTPStatus.UNAUTHORIZED:
            raise AuthenticationFailure()
        if response.status_code == HTTPStatus.NOT_FOUND:
            raise InvalidCpmHubUrl()
        if response.status_code == HTTPStatus.BAD_REQUEST:
            raise PublicationFailure("bad request")
        if response.status_code == HTTPStatus.CONFLICT:
            raise PublicationFailure(
                f'bit {project_descriptor.name}:{project_descriptor.version} has already been used and cannot be published again')
        if response.status_code != HTTPStatus.OK:
            raise PublicationFailure("cpm-hub returned error")

    def publish_template(self, project_descriptor, file_name):
        payload = base64.b64encode(filesystem.read_file(file_name, 'rb')).decode('utf-8')
        username = input('username: ')
        password = getpass(prompt='password: ', stream=None)
        body = {
            'template_name': project_descriptor.name,
            'version': project_descriptor.version,
            'payload': payload,
            'username': username,
            'password': password,
        }

        if self.dry_run:
            print(f'cpm: publishing to {self.repository_url}')
            print(json.dumps(body))
            return

        response = http_client.post(f'{self.repository_url}/templates', data=json.dumps(body))
        if response.status_code == HTTPStatus.UNAUTHORIZED:
            raise AuthenticationFailure()
        if response.status_code == HTTPStatus.NOT_FOUND:
            raise InvalidCpmHubUrl()
        if response.status_code == HTTPStatus.BAD_REQUEST:
            raise PublicationFailure("bad request")
        if response.status_code == HTTPStatus.CONFLICT:
            raise PublicationFailure(
                f'bit {project_descriptor.name}:{project_descriptor.version} has already been used and cannot be published again')
        if response.status_code != HTTPStatus.OK:
            raise PublicationFailure("cpm-hub returned error")

    def download_bit(self, name, version):
        response = http_client.get(self.__bit_url(name, version))
        if response.status_code == HTTPStatus.NOT_FOUND:
            raise BitNotFound(f'{name}:{version}')

        data = json.loads(response.body)
        return BitDownload(data['bit_name'], data['version'], data['payload'])

    def __bit_url(self, name, version):
        if version == "latest":
            return f'{self.repository_url}/bits/{name}'
        else:
            return f'{self.repository_url}/bits/{name}/{version}'

    def download_template(self, name, version):
        response = http_client.get(self.__template_url(name, version))
        if response.status_code == HTTPStatus.NOT_FOUND:
            raise TemplateNotFound(f'{name}:{version}')

        data = json.loads(response.body)
        return TemplateDownload(data['template_name'], data['version'], data['payload'])

    def __template_url(self, name, version):
        if version == "latest":
            return f'{self.repository_url}/templates/{name}'
        else:
            return f'{self.repository_url}/templates/{name}/{version}'


class AuthenticationFailure(RuntimeError):
    pass


class InvalidCpmHubUrl(RuntimeError):
    pass


class PublicationFailure(RuntimeError):
    pass


class BitNotFound(RuntimeError):
    pass


class TemplateNotFound(RuntimeError):
    pass



import glob
from pathlib import Path

import semver

from cpm.infrastructure.yaml_parser import YamlParser
from cpm.domain import constants
from cpm.domain.project.project_descriptor import ProjectDescriptor, TargetDescription, DeclaredBit, CompilationPlan, PackageDescription


def parse_from(project_directory):
    try:
        parser = YamlParser(pure=True)
        yaml_document = parser.load_from(Path(project_yaml_file(project_directory)))
    except FileNotFoundError:
        raise ProjectDescriptorNotFound
    project_descriptor = digest_yaml(yaml_document)
    project_descriptor.yaml_document = yaml_document
    project_descriptor.parser = parser
    return project_descriptor


def digest_yaml(yaml_contents):
    project_description = ProjectDescriptor()
    project_description.name = required(yaml_contents, 'name', String)
    project_description.version = optional(yaml_contents, 'version', String, '')
    project_description.description = optional(yaml_contents, 'description', String, '')
    build = optional(yaml_contents, 'build', Mapping, default={})
    project_description.build = parse_compilation_plan(build)
    test = optional(yaml_contents, 'test', Mapping, default={})
    project_description.test = parse_compilation_plan(test)
    project_description.targets = parse_targets(get_or_default_to(yaml_contents, 'targets', {}))
    return project_description


def parse_targets(targets_description):
    targets = {
        'default': TargetDescription('default')
    }
    for target_name in targets_description:
        targets[target_name] = parse_target(target_name, targets_description.get(target_name, {}))
    return targets


def parse_target(target_name, target_description):
    target = TargetDescription(target_name)
    target.image = target_description.get('image', '')
    target.dockerfile = target_description.get('dockerfile', '')
    target.test_image = target_description.get('test_image', '')
    target.test_dockerfile = target_description.get('test_dockerfile', '')
    target.toolchain_prefix = target_description.get('toolchain_prefix', '')
    target.format = target_description.get('format', 'binary')
    target.main = target_description.get('main', '')
    target.post_build = target_description.get('post_build', [])
    target.build = parse_compilation_plan(target_description.get('build', {}))
    target.test = parse_compilation_plan(target_description.get('test', {}))
    return target


def parse_compilation_plan(plan_description):
    compilation_plan = CompilationPlan()
    bits = optional(plan_description, 'bits', Mapping, default={})
    for bit_name in bits:
        bit_description = required(bits, bit_name, BitDescription)
        if isinstance(bit_description, str):
            declared_bit = DeclaredBit(bit_name, bits[bit_name])
        else:
            declared_bit = declared_bit_with_customized_compilation(bit_name, bit_description)
        compilation_plan.declared_bits.append(declared_bit)
    packages = optional(plan_description, 'packages', Mapping, default={})
    for package_path in packages:
        package_description = optional(packages, package_path, Mapping, {})
        package = PackageDescription(
            package_path,
            cflags=optional(package_description, 'cflags', sequence_of(String), []),
            cppflags=optional(package_description, 'cppflags', sequence_of(String), []),
        )
        compilation_plan.packages.append(package)
    compilation_plan.cflags = optional(plan_description, 'cflags', sequence_of(String), [])
    compilation_plan.cppflags = optional(plan_description, 'cppflags', sequence_of(String), [])
    compilation_plan.ldflags = optional(plan_description, 'ldflags', sequence_of(String), [])
    compilation_plan.libraries = optional(plan_description, 'libraries', sequence_of(String), [])
    compilation_plan.includes.update(optional(plan_description, 'includes', sequence_of(String), []))
    return compilation_plan


def declared_bit_with_customized_compilation(bit_name, bit_description):
    return DeclaredBit(
        name=bit_name,
        version=required(bit_description, 'version', String),
        cflags=optional(bit_description, 'cflags', Sequence, []),
        cppflags=optional(bit_description, 'cppflags', Sequence, []),
        target=optional(bit_description, 'target', String, '')
    )


def project_yaml_file(project_directory):
    return f'{project_directory}/{constants.PROJECT_DESCRIPTOR_FILE}'


def all_bit_yaml_files(project_descriptor):
    return glob.glob(f'{project_descriptor}/bits/*/*.yaml')


def package_compilation_flags(package_description, field):
    return get_or_default_to(package_description, field, [])


def get_or_default_to(dictionary, key, default):
    if not dictionary:
        return default
    return dictionary.get(key, default) or default


def required(d, key, typ):
    if key not in d:
        raise MissingRequiredField(key)
    typ.validate(d, key)
    return d[key]


def optional(d, key, typ, default=None):
    if key not in d or not d[key]:
        return default
    typ.validate(d, key)
    return d[key]


class String:
    name = 'string'

    @staticmethod
    def validate(d, key):
        value = d[key]
        if not isinstance(value, str):
            raise ParseError(*_location(d, key), f'{key} must be a string')


class Semver(String):
    name = 'semver'

    @staticmethod
    def validate(d, key):
        String.validate(d, key)
        value = d[key]
        if not semver.VersionInfo.isvalid(value):
            raise ParseError(*_location(d, key), f'{key} must be a valid semver string')


class BitDescription:
    name = 'bit_description'

    @staticmethod
    def validate(d, key):
        value = d[key]
        if not isinstance(value, dict) and not isinstance(value, str):
            raise ParseError(*_location(d, key), f'{key} must be a string or a mapping')


class Mapping:
    name = 'mapping'

    @staticmethod
    def validate(d, key):
        value = d[key]
        if not isinstance(value, dict):
            raise ParseError(*_location(d, key), f'{key} must be a mapping')


class Sequence:
    name = 'sequence'

    @staticmethod
    def validate(d, key):
        sequence = d[key]
        if not isinstance(sequence, list):
            raise ParseError(*_location(d, key), f'{key} must be a sequence')


def sequence_of(typ):
    class SequenceOf(Sequence):
        name = f'sequence_of_{typ.name}'

        @staticmethod
        def validate(d, key):
            try:
                Sequence.validate(d, key)
                sequence = d[key]
                for i in range(len(sequence)):
                    typ.validate(sequence, i)
            except ParseError as parse_error:
                raise ParseError(
                    parse_error.parsing_file,
                    parse_error.line,
                    parse_error.col,
                    f'{key} must be a sequence of {typ.name}')

    return SequenceOf


class ProjectDescriptorNotFound(RuntimeError):
    pass


def _location(d, key):
    return d.parsing_file, d.lc.data[key][0], d.lc.data[key][1]


class ParseError(RuntimeError):
    def __init__(self, parsing_file, line, col, message):
        self.parsing_file = parsing_file
        self.line = line
        self.col = col
        self.message = f'{parsing_file}:{line+1}:{col+1}: {message}'


class MissingRequiredField(ParseError):
    def __init__(self, field):
        self.field = field
        self.message = f'{field} is required'



from collections import OrderedDict


class CMakeBuilder(object):
    def __init__(self):
        self.contents = ''

    def minimum_required(self, version):
        self.contents += f'cmake_minimum_required (VERSION {version})\n'
        return self

    def project(self, name):
        self.contents += f'project({name})\n'
        return self

    def include(self, directories):
        self.contents += f'include_directories({" ".join(self.no_duplicates(directories))})\n'
        return self

    def no_duplicates(self, directories):
        return list(OrderedDict.fromkeys(directories))

    def set_source_files_properties(self, sources, property, values):
        self.contents += f'set_source_files_properties({" ".join(sources)} PROPERTIES {property} "{" ".join(values)}")\n'
        return self

    def add_object_library(self, name, sources):
        self.contents += f'add_library({name} OBJECT {" ".join(sources)})\n'
        return self

    def add_static_library(self, name, sources):
        self.contents += f'add_library({name} STATIC {" ".join(sources)})\n'
        return self

    def add_executable(self, name, sources, object_libraries=[]):
        self.contents += f'add_executable({name} {" ".join(sources)}'
        for library in object_libraries:
            self.contents += f' $<TARGET_OBJECTS:{library}>'
        self.contents += ')\n'
        return self

    def set_target_properties(self, target, property, values):
        self.contents += f'set_target_properties({target} PROPERTIES {property} "{" ".join(values)}")\n'
        return self

    def target_link_libraries(self, target, libraries):
        self.contents += f'target_link_libraries({target} {" ".join(libraries)})\n'
        return self

    def target_include_directories(self, target, directories):
        self.contents += f'target_include_directories({target} PUBLIC {" ".join(directories)})\n'
        return self

    def add_custom_target(self, target, command, depends):
        self.contents += f'add_custom_target({target}\n' \
                         f'    COMMAND {command}"\n' \
                         f'    DEPENDS {" ".join(depends)}\n' \
                         f')\n'
        return self

    def add_custom_command(self, target, when, command):
        self.contents += f'add_custom_command(\n' \
                         f'    TARGET {target}\n' \
                         f'    {when}\n' \
                         f'    COMMAND COMMAND {command}\n' \
                         f')\n'
        return self


def a_cmake():
    return CMakeBuilder()



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
from cpm.domain.compilation_service import CompilationService
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.domain.project.project_loader import ProjectLoader
from cpm.domain.project_commands import ProjectCommands


def prep_project(compilation_service, target='default'):
    try:
        compilation_service.update(target)
    except ProjectDescriptorNotFound:
        return Result(FAIL, f'error: not a cpm project')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, f'CMakeLists.txt ready')


def execute(argv):
    create_parser = argument_parser()
    args = create_parser.parse_args(argv)

    project_loader = ProjectLoader()
    cmakelists_builder = CMakeListsBuilder()
    project_commands = ProjectCommands()
    service = CompilationService(project_loader, cmakelists_builder, project_commands)

    result = prep_project(service, args.target)

    return result


def argument_parser():
    create_parser = ArgumentParser(prog='cpm prep', description=description())
    create_parser.add_argument('target',
                               nargs='?',
                               help='target to to use when generating the CMakeLists.txt file',
                               default='default')
    return create_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'generate the CMakeLists.txt file for the given target (use \'default\' if none is given)'



import base64

from cpm.infrastructure import filesystem
from cpm.domain.constants import bit_directory


class BitInstaller(object):
    def __init__(self, project_loader):
        self.project_loader = project_loader

    def install(self, bit_download):
        directory = bit_directory(bit_download.bit_name, bit_download.version)
        if filesystem.directory_exists(directory):
            filesystem.remove_directory(directory)
        filesystem.create_directory(directory)
        filesystem.unzips(base64.b64decode(bit_download.payload), directory)



from cpm.domain.project import project_descriptor_parser
from cpm.infrastructure.cpm_hub_connector_v1 import BitNotFound
from cpm.infrastructure.http_client import HttpConnectionError
from cpm.domain.constants import bit_directory


class InstallService(object):
    def __init__(self, project_loader, bit_installer, cpm_hub_connector):
        self.project_loader = project_loader
        self.cpm_hub_connector = cpm_hub_connector
        self.bit_installer = bit_installer

    def install(self, name, version):
        try:
            if not self.__bit_already_installed(name, version):
                bit_download = self.cpm_hub_connector.download_bit(name, version)
                self.bit_installer.install(bit_download)
                print(f'  {f"{name}:{version}": <20} {"✔": >20}')
        except BitNotFound:
            print(f'  {f"{name}:{version}": <20} {f"✖ bit {name} not found in bits repository": >20}')
        except HttpConnectionError as error:
            print(f'  {f"{name}:{version}": <20} {f"✖ failed to connect to bits repository: {error}": >20}')

    def __bit_already_installed(self, name, version):
        try:
            bit_description = project_descriptor_parser.parse_from(bit_directory(name, version))
            return bit_description.version == version
        except:
            return False

    def _log_install_or_upgrade(self, project_description, name, version):
        installed_bit = next((bit for bit in project_description.build.bits if bit.name == name), None)
        if installed_bit:
            print(f'  {"upgrading" if installed_bit.version < version else "downgrading"} {name}:{installed_bit.version} -> {version}')

    def install_all(self, directory='.'):
        print(f'cpm: updating dependencies')
        self.__install_recursively(directory)
        project_descriptor = project_descriptor_parser.parse_from(directory)
        for declared_bit in project_descriptor.test.declared_bits:
            self.install(declared_bit.name, declared_bit.version)
        print(f'cpm: everything up to date')

    def __install_recursively(self, directory='.'):
        project_descriptor = project_descriptor_parser.parse_from(directory)
        for declared_bit in project_descriptor.build.declared_bits:
            self.install(declared_bit.name, declared_bit.version)
            self.__install_recursively(bit_directory(declared_bit.name, declared_bit.version))



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.publish_service import PublishService
from cpm.domain.project_packager import PackagingFailure
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.infrastructure.cpm_hub_connector_v1 import CpmHubConnectorV1
from cpm.infrastructure.cpm_hub_connector_v1 import InvalidCpmHubUrl
from cpm.infrastructure.cpm_hub_connector_v1 import AuthenticationFailure
from cpm.infrastructure.cpm_hub_connector_v1 import PublicationFailure
from cpm.infrastructure.http_client import HttpConnectionError


def publish_project(publish_service, publish_as_template=False):
    try:
        if publish_as_template:
            publish_service.publish_template()
        else:
            publish_service.publish()
    except ProjectDescriptorNotFound:
        return Result(FAIL, f'error: not a CPM project')
    except PackagingFailure as error:
        return Result(FAIL, f'error: {error.cause}')
    except HttpConnectionError:
        return Result(FAIL, f'error: failed to connect to CPM Hub')
    except InvalidCpmHubUrl:
        return Result(FAIL, f'error: invalid CPM Hub URL')
    except AuthenticationFailure:
        return Result(FAIL, f'error: invalid credentials')
    except PublicationFailure as error:
        return Result(FAIL, f'error: {error}')
    except KeyboardInterrupt:
        return Result(FAIL, f'interrupted')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, f'Project published')


def execute(argv):
    publish_parser = argument_parser()
    args = publish_parser.parse_args(argv)

    cpm_hub_connector = CpmHubConnectorV1(repository_url=args.repository_url, dry_run=args.dry_run)
    service = PublishService(cpm_hub_connector)

    result = publish_project(service, publish_as_template=args.template)

    return result


def argument_parser():
    publish_parser = ArgumentParser(prog='cpm publish', description=description())
    publish_parser.add_argument('-s', '--repository-url',
                                required=True,
                                action='store',
                                help='URL of the cpm repository (default https://repo.cpmbits.com:8000)',
                                default='https://repo.cpmbits.com:8000')
    publish_parser.add_argument('-t', '--template',
                                required=False,
                                action='store_true',
                                help='publish the project as a template',
                                default=False)
    publish_parser.add_argument('-d', '--dry-run',
                                required=False,
                                action='store_true',
                                help='package the project and try login but do not publish it yet',
                                default=False)
    return publish_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'publish the project'



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.install_service import InstallService
from cpm.domain.bit_installer import BitInstaller
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.domain.project.project_loader import ProjectLoader
from cpm.infrastructure.cpm_hub_connector_v1 import CpmHubConnectorV1, BitNotFound
from cpm.infrastructure.cpm_user_configuration import CpmUserConfiguration
from cpm.infrastructure.http_client import HttpConnectionError


def install_bit(install_service, name, version='latest'):
    try:
        install_service.install(name, version)
    except ProjectDescriptorNotFound:
        return Result(FAIL, 'error: not a cpm project')
    except BitNotFound:
        return Result(FAIL, f'error: bit {name} not found in CPM Hub')
    except HttpConnectionError as error:
        return Result(FAIL, f'error: failed to connect to CPM Hub at {error}')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, f'installed bit {name}:{version}')


def install_project_bits(install_service):
    try:
        install_service.install_all()
    except ProjectDescriptorNotFound:
        return Result(FAIL, 'error: not a cpm project')
    except BitNotFound as e:
        return Result(FAIL, f'error: bit {e} not found in CPM Hub')
    except HttpConnectionError as error:
        return Result(FAIL, f'error: failed to connect to CPM Hub at {error}')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, f'installed bits')


def execute(argv):
    install_bit_arg_parser = argument_parser()
    args = install_bit_arg_parser.parse_args(argv)

    project_loader = ProjectLoader()
    bit_installer = BitInstaller(project_loader)
    user_configuration = CpmUserConfiguration()
    user_configuration.load()
    repository_url = args.repository_url if args.repository_url else user_configuration["cpm_hub_url"]
    cpm_hub_connector = CpmHubConnectorV1(repository_url=f'{repository_url}')
    service = InstallService(project_loader, bit_installer, cpm_hub_connector)
    bit_name, bit_version = __bit_to_install(args.bit_to_install)

    if not args.bit_to_install:
        result = install_project_bits(service)
    else:
        result = install_bit(service, bit_name, bit_version)

    return result


def argument_parser():
    install_bit_arg_parser = ArgumentParser(prog='cpm install', description=description())
    install_bit_arg_parser.add_argument('-s', '--repository-url',
                                        required=False,
                                        action='store',
                                        arg_format='<repository_url>',
                                        help='URL of the cpm repository (default https://repo.cpmbits.com:8000)',
                                        default='https://repo.cpmbits.com:8000')
    install_bit_arg_parser.add_argument('bit_to_install',
                                        help='bit to install',
                                        arg_format='<name>:<version>',
                                        nargs='?')
    return install_bit_arg_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'install bits'


def __bit_to_install(bit_argument):
    parts = f'{bit_argument}:latest'.split(':')
    return parts[0], parts[1]



import unittest
import mock

from cpm.domain.template_download import TemplateDownload
from cpm.domain.template_installer import TemplateInstaller, UnableToInstallTemplate


class TestTemplateInstaller(unittest.TestCase):
    @mock.patch('cpm.domain.template_installer.filesystem')
    def test_template_installation_when_template_was_not_installed_before(self, filesystem):
        installer = TemplateInstaller()
        template_download = TemplateDownload("arduino", "1.0", "dGVtcGxhdGUgZG93bmxvYWQ=")
        filesystem.directory_exists.return_value = False

        installer.install(template_download, './project')

        filesystem.create_directory.assert_called_once_with('./project')
        filesystem.unzips.assert_called_once_with(b'template download', './project')

    @mock.patch('cpm.domain.template_installer.filesystem')
    def test_template_installation_raises_an_error_when_directory_sxists(self, filesystem):
        installer = TemplateInstaller()
        template_download = TemplateDownload("arduino", "1.0", "dGVtcGxhdGUgZG93bmxvYWQ=")
        filesystem.directory_exists.return_value = True

        self.assertRaises(UnableToInstallTemplate, installer.install, template_download, './project')




import threading
import pytest

from test.e2e.environment import cpmhub_mock
from cpm.infrastructure import http_client
from werkzeug.serving import make_server


server = None


class ServerThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.srv = make_server('127.0.0.1', 8000, cpmhub_mock.app)
        self.ctx = cpmhub_mock.app.app_context()
        self.ctx.push()

    def run(self):
        self.srv.serve_forever()

    def shutdown(self):
        self.srv.shutdown()


def start_server():
    global server
    server = ServerThread()
    server.start()


def stop_server():
    global server
    server.shutdown()


@pytest.fixture(scope="module", autouse=True)
def start_core_mock():
    start_server()
    __wait_for_server()
    yield
    stop_server()


def __wait_for_server():
    alive = False
    while not alive:
        try:
            http_client.get('http://localhost:8000/')
            alive = True
        except Exception as e:
            pass



import unittest
from mock import patch

from http import HTTPStatus

from cpm.domain.bit_download import BitDownload
from cpm.domain.project.project import Project
from cpm.domain.template_download import TemplateDownload
from cpm.infrastructure.cpm_hub_connector_v1 import CpmHubConnectorV1, BitNotFound, TemplateNotFound
from cpm.infrastructure.cpm_hub_connector_v1 import AuthenticationFailure, InvalidCpmHubUrl, PublicationFailure
from cpm.infrastructure.http_client import HttpResponse


class TestCpmHubConnectorV1(unittest.TestCase):
    @patch('cpm.infrastructure.cpm_hub_connector_v1.filesystem')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    @patch('builtins.input')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    def test_publish_bit_posts_bit_with_base64_encoded_payload_and_user_credentials(self, getpass, input, http_client, filesystem):
        project = Project('cpm-hub')
        connector = CpmHubConnectorV1()
        http_client.post.return_value = HttpResponse(200, '')
        filesystem.read_file.return_value = b'bit payload'
        getpass.return_value = 'password'
        input.return_value = 'username'

        connector.publish_bit(project, 'cpm-hub.zip')

        http_client.post.assert_called_once_with(
            f'{connector.repository_url}/bits',
            data='{"bit_name": "cpm-hub", "version": "0.1", "payload": "Yml0IHBheWxvYWQ=", "username": "username", "password": "password"}'
        )

    @patch('cpm.infrastructure.cpm_hub_connector_v1.filesystem')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    @patch('builtins.input')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    def test_publish_bit_raises_authentication_error_on_status_code_401(self, getpass, input, http_client, filesystem):
        project = Project('cpm-hub')
        connector = CpmHubConnectorV1()
        http_client.post.return_value = HttpResponse(401, '')
        filesystem.read_file.return_value = b'bit payload'
        getpass.return_value = 'password'
        input.return_value = 'username'

        self.assertRaises(AuthenticationFailure, connector.publish_bit, project, 'cpm-hub.zip')

    @patch('cpm.infrastructure.cpm_hub_connector_v1.filesystem')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    @patch('builtins.input')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    def test_publish_bit_raises_invalid_cpm_hub_url_on_status_code_404(self, getpass, input, http_client, filesystem):
        project = Project('cpm-hub')
        connector = CpmHubConnectorV1()
        http_client.post.return_value = HttpResponse(404, '')
        filesystem.read_file.return_value = b'bit payload'
        getpass.return_value = 'password'
        input.return_value = 'username'

        self.assertRaises(InvalidCpmHubUrl, connector.publish_bit, project, 'cpm-hub.zip')

    @patch('cpm.infrastructure.cpm_hub_connector_v1.filesystem')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    @patch('builtins.input')
    @patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    def test_publish_bit_raises_publication_error_when_status_code_is_not_200(self, getpass, input, http_client, filesystem):
        project = Project('cpm-hub')
        connector = CpmHubConnectorV1()
        http_client.post.return_value = HttpResponse(409, '')
        filesystem.read_file.return_value = b'bit payload'
        getpass.return_value = 'password'
        input.return_value = 'username'

        self.assertRaises(PublicationFailure, connector.publish_bit, project, 'cpm-hub.zip')

    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    def test_download_bit_gets_bit_with_base64_encoded_payload(self, http_client):
        connector = CpmHubConnectorV1()
        http_client.get.return_value = HttpResponse(HTTPStatus.OK, '{"bit_name": "cpm-hub", "version": "0.1", "payload": "Yml0IHBheWxvYWQ="}')

        bit_download = connector.download_bit('cest', 'latest')

        http_client.get.assert_called_once_with(f'{connector.repository_url}/bits/cest')
        assert bit_download == BitDownload("cpm-hub", "0.1", "Yml0IHBheWxvYWQ=")

    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    def test_download_bit_gets_bit_with_specific_version(self, http_client):
        connector = CpmHubConnectorV1()
        http_client.get.return_value = HttpResponse(HTTPStatus.OK, '{"bit_name": "cpm-hub", "version": "1.0", "payload": "Yml0IHBheWxvYWQ="}')

        bit_download = connector.download_bit('cest', '1.0')

        http_client.get.assert_called_once_with(f'{connector.repository_url}/bits/cest/1.0')
        assert bit_download == BitDownload("cpm-hub", "1.0", "Yml0IHBheWxvYWQ=")

    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    def test_download_bit_raises_exception_when_bit_is_not_found(self, http_client):
        connector = CpmHubConnectorV1()
        http_client.get.return_value = HttpResponse(HTTPStatus.NOT_FOUND, '')

        self.assertRaises(BitNotFound, connector.download_bit, 'cest', 'latest')

        http_client.get.assert_called_once_with(f'{connector.repository_url}/bits/cest')

    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    def test_download_template_gets_bit_with_base64_encoded_payload(self, http_client):
        connector = CpmHubConnectorV1()
        http_client.get.return_value = HttpResponse(HTTPStatus.OK, '{"template_name": "arduino", "version": "1.0.0", "payload": "Yml0IHBheWxvYWQ="}')

        template_download = connector.download_template('arduino', 'latest')

        http_client.get.assert_called_once_with(f'{connector.repository_url}/templates/arduino')
        assert template_download == TemplateDownload("arduino", "1.0.0", "Yml0IHBheWxvYWQ=")

    @patch('cpm.infrastructure.cpm_hub_connector_v1.http_client')
    def test_download_template_raises_exception_when_template_is_not_found(self, http_client):
        connector = CpmHubConnectorV1()
        http_client.get.return_value = HttpResponse(HTTPStatus.NOT_FOUND, '')

        self.assertRaises(TemplateNotFound, connector.download_template, 'arduino', 'latest')

        http_client.get.assert_called_once_with(f'{connector.repository_url}/templates/arduino')



import os
import shutil
import zipfile
import io
from distutils.dir_util import copy_tree
from pathlib import Path


def create_file(file_name, contents=''):
    with open(file_name, 'w') as f:
        f.write(contents)


def write_file(file_name, contents=''):
    with open(file_name, 'w') as f:
        f.write(contents)


def read_file(file_name, mode='r'):
    with open(file_name, mode) as file_stream:
        return file_stream.read()


def file_exists(name):
    return os.path.exists(name) and os.path.isfile(name)


def copy_file(origin, destination):
    shutil.copy2(origin, destination)


def delete_file(file_name):
    os.remove(file_name)


def create_directory(path):
    os.makedirs(path)


def copy_directory(origin, destination):
    copy_tree(origin, destination)


def path_to(filename):
    return os.path.dirname(filename)


def directory_exists(name):
    return os.path.exists(name) and os.path.isdir(name)


def is_directory(path):
    return os.path.isdir(path)


def is_file(path):
    return os.path.isfile(path)


def parent_directory(path):
    return str(Path(path).parent)


def remove_directory(path):
    shutil.rmtree(path)


def list_directories(path):
    if directory_exists(path):
        return next(os.walk(path))[1]
    return []


def home_directory():
    return str(Path.home())


def find(path, pattern):
    return [str(filename) for filename in Path(path).rglob(pattern)]


def zip(directory, output_filename):
    shutil.make_archive(output_filename, 'zip', directory)


def unzips(payload, directory):
    zip = zipfile.ZipFile(io.BytesIO(payload))
    zip.extractall(path=directory)
    zip.close()


def symlink(source, destination):
    if Path(destination).is_symlink():
        return
    os.symlink(source, destination, target_is_directory=Path(source).is_dir())



from cpm.domain.project.project_descriptor_parser import ParseError


class CompilationService(object):
    def __init__(self, project_loader, cmakelists_builder, project_commands):
        self.project_loader = project_loader
        self.cmakelists_builder = cmakelists_builder
        self.project_commands = project_commands

    def build(self, target_name='default'):
        project = self.project_loader.load('.', target_name)
        self.cmakelists_builder.build(project)
        self.project_commands.build(project)

    def update(self, target_name='default'):
        project = self.project_loader.load('.', target_name)
        self.cmakelists_builder.build(project)

    def clean(self):
        project = self.try_load_project()
        self.project_commands.clean(project)

    def try_load_project(self):
        project = None
        try:
            project = self.project_loader.load('.')
        except ParseError as e:
            print(f'cpm: warning: parsing error while cleaning ({e.message})')
        return project



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result, OK, FAIL
from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
from cpm.domain.compilation_service import CompilationService
from cpm.domain.project_commands import DockerImageNotFound
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.domain.project.project_loader import ProjectLoader, InvalidTarget
from cpm.domain.project_commands import ProjectCommands, BuildError
from cpm.api import install


def build_project(compilation_service, target='default'):
    try:
        print(f'cpm: building target {target}')
        compilation_service.build(target)
    except ProjectDescriptorNotFound:
        return Result(FAIL, f'error: not a cpm project')
    except BuildError:
        return Result(FAIL, f'error: compilation failed')
    except InvalidTarget:
        return Result(FAIL, f'error: unknown target {target}')
    except DockerImageNotFound as e:
        return Result(FAIL, f'error: docker image {e.image_name} not found for target {target}')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, f'Build finished')


def execute(argv):
    install.execute([])

    create_parser = argument_parser()
    args = create_parser.parse_args(argv)

    project_loader = ProjectLoader()
    cmakelists_builder = CMakeListsBuilder()
    project_commands = ProjectCommands()
    service = CompilationService(project_loader, cmakelists_builder, project_commands)

    result = build_project(service, args.target)

    return result


def argument_parser():
    create_parser = ArgumentParser(prog='cpm build', description=description())
    create_parser.add_argument('target',
                               help='target to build',
                               nargs='?',
                               default='default')
    return create_parser


def print_help():
    return argument_parser().print_help()


def description():
    return 'build the project for the given target (use \'default\' if none is given)'



import argparse
from collections import defaultdict
import textwrap


class LineWrapRawTextHelpFormatter(argparse.RawDescriptionHelpFormatter):
    def __init__(self, *args, options=[], **kwargs):
        self.options = options
        super(LineWrapRawTextHelpFormatter, self).__init__(*args, **kwargs)

    def _split_lines(self, text, width):
        text = self._whitespace_matcher.sub(' ', text).strip()
        return textwrap.wrap(text, width)

    def _get_default_metavar_for_optional(self, action):
        for option in self.options:
            if action.option_strings == option['flags'] and option['arg_format'] is not None:
                return option['arg_format']
        return action.dest

    def _get_default_metavar_for_positional(self, action):
        for option in self.options:
            if action.dest == option['flags'][0] and option['arg_format'] is not None:
                return option['arg_format']
        return action.dest


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        self.program = {
            key: kwargs[key] for key in kwargs
        }
        self.options = []
        super(ArgumentParser, self).__init__(*args, **kwargs)

    def add_argument(self, *args, arg_format=None, **kwargs):
        super(argparse.ArgumentParser, self).add_argument(*args, **kwargs)
        option = defaultdict(str)
        option['flags'] = [item for item in args]
        for key in kwargs:
            option['arg_format'] = arg_format
            option[key] = kwargs[key]
        self.options.append(option)

    def _get_formatter(self):
        return LineWrapRawTextHelpFormatter(prog=self.prog, options=self.options)



import unittest
import mock

from cpm.api.build import build_project
from cpm.domain.project.project_loader import InvalidTarget
from cpm.domain.project_commands import DockerImageNotFound, BuildError
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError


class TestApiBuild(unittest.TestCase):
    def test_build_fails_when_current_directory_is_not_a_cpm_project(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.build.side_effect = ProjectDescriptorNotFound()

        result = build_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_fails_when_descriptor_contains_errors(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.build.side_effect = ParseError('project.yaml', 1, 1, 'error')

        result = build_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_fails_when_compilation_fails(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.build.side_effect = BuildError()

        result = build_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_fails_when_target_is_unknown(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.build.side_effect = InvalidTarget()

        result = build_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_fails_when_docker_image_is_not_found(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.build.side_effect = DockerImageNotFound('')

        result = build_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_project_for_host(self):
        compilation_service = mock.MagicMock()
        recipe = mock.MagicMock()

        result = build_project(compilation_service, recipe)

        assert result.status_code == 0
        compilation_service.build.assert_called_once_with(recipe)

    def test_build_project_for_target(self):
        compilation_service = mock.MagicMock()

        result = build_project(compilation_service, target='raspberrypi4:64')

        assert result.status_code == 0
        compilation_service.build.assert_called_once_with('raspberrypi4:64')



PROJECT_DESCRIPTOR_FILE = 'project.yaml'
BUILD_DIRECTORY = 'build'
CMAKELISTS = 'CMakeLists.txt'
CMAKE_COMMAND = 'cmake'
NINJA_COMMAND = 'ninja'
INITIAL_PROJECT_VERSION = '0.1.0'
DEFAULT_TARGET = 'default'


def bit_directory(name, version):
    return f'bits/{name}/{version}'



class TestService(object):
    def __init__(self, project_loader, cmakelists_builder, project_commands):
        self.project_loader = project_loader
        self.cmakelists_builder = cmakelists_builder
        self.project_commands = project_commands

    def run_tests(self, files_or_dirs, target_name, test_args=()):
        project = self.project_loader.load('.', target_name)
        if not project.test.test_suites:
            raise NoTestsFound()
        self.cmakelists_builder.build(project)
        self.project_commands.build_tests(project, files_or_dirs)
        self.project_commands.run_tests(project, files_or_dirs, test_args)


class NoTestsFound(RuntimeError):
    pass



import unittest
import mock

from cpm.infrastructure.cpm_user_configuration import CpmUserConfiguration
from cpm.infrastructure.cpm_user_configuration import GLOBAL_CONFIGURATION_FILENAME


class TestCpmUserConfiguration(unittest.TestCase):
    @mock.patch('cpm.infrastructure.cpm_user_configuration.filesystem')
    @mock.patch('cpm.infrastructure.cpm_user_configuration.yaml_parser')
    def test_loading_configuration_uses_defaults_when_global_configuration_file_does_not_exist(self, yaml_parser, filesystem):
        filesystem.file_exists.return_value = False
        filesystem.home_directory.return_value = '/home/cpmuser'
        cpm_user_configuration = CpmUserConfiguration()

        cpm_user_configuration.load()

        assert cpm_user_configuration['cpm_hub_url'] == 'https://repo.cpmbits.com:8000'
        yaml_parser.load.assert_not_called()
        filesystem.file_exists.assert_called_once_with(f'/home/cpmuser/{GLOBAL_CONFIGURATION_FILENAME}')

    @mock.patch('cpm.infrastructure.cpm_user_configuration.filesystem')
    @mock.patch('cpm.infrastructure.cpm_user_configuration.yaml_parser')
    def test_loading_configuration_loads_defaults_on_empty_global_configuration(self, yaml_parser, filesystem):
        yaml_parser.load.return_value = {}
        filesystem.file_exists.return_value = True
        filesystem.home_directory.return_value = '/home/cpmuser'
        cpm_user_configuration = CpmUserConfiguration()

        cpm_user_configuration.load()

        assert cpm_user_configuration['cpm_hub_url'] == 'https://repo.cpmbits.com:8000'
        yaml_parser.load.assert_called_once_with(f'/home/cpmuser/{GLOBAL_CONFIGURATION_FILENAME}')

    @mock.patch('cpm.infrastructure.cpm_user_configuration.filesystem')
    @mock.patch('cpm.infrastructure.cpm_user_configuration.yaml_parser')
    def test_loading_configuration_updates_defaults_with_global_configuration_values(self, yaml_parser, filesystem):
        filesystem.file_exists.return_value = True
        yaml_parser.load.return_value = {'cpm_hub_url': 'http://otherserver.com:8000'}
        cpm_user_configuration = CpmUserConfiguration()

        cpm_user_configuration.load()

        assert cpm_user_configuration['cpm_hub_url'] == 'http://otherserver.com:8000'




import argparse

from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
from cpm.domain.project.project_loader import ProjectLoader
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.domain.test_service import TestService
from cpm.domain.test_service import NoTestsFound
from cpm.domain.project_commands import BuildError
from cpm.domain.project_commands import TestsFailed
from cpm.domain.project_commands import ProjectCommands


def run_tests(test_service, files_or_dirs=(), test_args=(), target='default'):
    try:
        test_service.run_tests(files_or_dirs, target, test_args)
    except ProjectDescriptorNotFound:
        return Result(FAIL, 'error: not a cpm project')
    except BuildError as e:
        return Result(FAIL, f'error: failed building tests')
    except TestsFailed:
        return Result(FAIL, '✖ FAIL')
    except NoTestsFound:
        return Result(OK, 'no tests to run')
    except ParseError as e:
        return Result(FAIL, f'error: {e.message}')

    return Result(OK, '✔ PASS')


def execute(argv):
    add_target_parser = argument_parser()
    args = add_target_parser.parse_args(argv)

    project_loader = ProjectLoader()
    cmakelists_builder = CMakeListsBuilder()
    project_commands = ProjectCommands()
    service = TestService(project_loader, cmakelists_builder, project_commands)
    files_or_dirs, test_args = __parse_rest(args.rest)

    result = run_tests(service, files_or_dirs, test_args)

    return result


def argument_parser():
    add_target_parser = ArgumentParser(prog='cpm test',
                                       usage='cpm test [<files or dirs>] [-- test_args]',
                                       description=description())
    add_target_parser.add_argument('rest',
                                   help='rest of the arguments will be passed as command line arguments in the tests',
                                   nargs=argparse.REMAINDER)
    return add_target_parser


def print_help():
    parser = argument_parser()
    parser.print_usage()
    print()
    print(parser.description)
    print()
    print('positional arguments:')
    print('  <files or dirs>\ta list of files or directories to test')
    print('  -- <test_args>\tanything after the -- will be passed as command line arguments to the tests')


def description():
    return 'build and run the project tests'


def __parse_rest(rest):
    try:
        delimiter_index = rest.index('--')
        files_or_dirs = rest[:delimiter_index]
        test_args = rest[delimiter_index+1:]
    except ValueError:
        files_or_dirs = rest
        test_args = []
    return files_or_dirs, test_args



import unittest

from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
from cpm.domain.project.project import Project, Target, Package, Test, TestSuite


def a_project(name):
    return TestProjectBuilder(name)


class TestCmakelistsBuilder(unittest.TestCase):
    def test_build_cmakelists_from_project(self):
        cmakelists_builder = CMakeListsBuilder()
        bit = a_project('Bit') \
            .with_target('default') \
            .with_package('bit_package', ['bit.cpp', 'bit.c'], ['-DBIT_HELLO']) \
            .project.target
        test_bit = a_project('TestBit') \
            .with_target('default') \
            .with_package('test_bit_package', ['test_bit.cpp', 'test_bit.c'], ['-DTEST_BIT_HELLO']) \
            .project.target
        project = a_project('Project') \
            .with_target('default') \
            .with_cflags(['-std=c99']) \
            .with_cppflags(['-std=c++11']) \
            .with_ldflags(['-Wl,--wrap=malloc']) \
            .with_libraries(['pthread']) \
            .with_package('package', ['file.cpp', 'file.c'], ['-DHOLA']) \
            .with_package('spdlog', [], []) \
            .with_test('test_case') \
            .with_bits([bit]) \
            .with_test_bits([test_bit]) \
            .with_test_package('bits/cest', [], []) \
            .with_test_package('bits/mock', ['bits/mock/mock.cpp'], []) \
            .with_test_ldflags(['-Wl,--allow-multiple-definition']) \
            .with_toolchain_prefix('arm-linux-gnueabi-') \
            .sort_include_directories() \
            .project

        cmakelists_content = cmakelists_builder.build_contents(project)

        print(cmakelists_content)

        assert 'set(CMAKE_C_COMPILER arm-linux-gnueabi-gcc)' in cmakelists_content
        assert 'set(CMAKE_CXX_COMPILER arm-linux-gnueabi-g++)' in cmakelists_content
        assert 'project(Project)' in cmakelists_content
        assert 'add_library(bit_package_c_object_library OBJECT bit.c)' in cmakelists_content
        assert 'set_target_properties(bit_package_c_object_library PROPERTIES COMPILE_FLAGS "-DBIT_HELLO")' in cmakelists_content
        assert 'add_library(bit_package_cpp_object_library OBJECT bit.cpp)' in cmakelists_content
        assert 'add_library(package_c_object_library OBJECT file.c)' in cmakelists_content
        assert 'set_target_properties(package_c_object_library PROPERTIES COMPILE_FLAGS "-DHOLA")' in cmakelists_content
        assert 'add_library(package_cpp_object_library OBJECT file.cpp)' in cmakelists_content
        assert 'link_libraries(pthread)' in cmakelists_content
        assert 'add_executable(Project main.cpp $<TARGET_OBJECTS:bit_package_c_object_library> $<TARGET_OBJECTS:bit_package_cpp_object_library> $<TARGET_OBJECTS:package_c_object_library> $<TARGET_OBJECTS:package_cpp_object_library>)' in cmakelists_content
        assert 'set_target_properties(Project PROPERTIES COMPILE_FLAGS "-std=c++11")' in cmakelists_content
        assert 'target_link_options(Project PUBLIC "-Wl,--wrap=malloc")' in cmakelists_content
        assert 'include_directories(package spdlog)' in cmakelists_content
        assert 'add_library(bits_mock_cpp_object_library OBJECT bits/mock/mock.cpp)' in cmakelists_content
        assert 'add_library(test_bit_package_c_object_library OBJECT test_bit.c)' in cmakelists_content
        assert 'set_target_properties(test_bit_package_c_object_library PROPERTIES COMPILE_FLAGS "-DTEST_BIT_HELLO")' in cmakelists_content
        assert 'add_library(test_bit_package_cpp_object_library OBJECT test_bit.cpp)' in cmakelists_content
        assert 'add_executable(test_case test_case.cpp $<TARGET_OBJECTS:bit_package_c_object_library> $<TARGET_OBJECTS:bit_package_cpp_object_library> $<TARGET_OBJECTS:package_c_object_library> $<TARGET_OBJECTS:package_cpp_object_library> $<TARGET_OBJECTS:bits_mock_cpp_object_library> $<TARGET_OBJECTS:test_bit_package_c_object_library> $<TARGET_OBJECTS:test_bit_package_cpp_object_library>)' in cmakelists_content
        assert 'target_link_options(test_case PUBLIC "-Wl,--allow-multiple-definition")' in cmakelists_content
        assert 'target_include_directories(test_case PUBLIC bits/cest bits/mock)' in cmakelists_content
        assert ('add_custom_target(tests\n'
                '    COMMAND echo ""\n'
                '    DEPENDS test_case\n'
                ')\n') in cmakelists_content


class TestProjectBuilder:
    def __init__(self, name):
        self.target_name = ''
        self.target = None
        self.project = Project(name)

    def with_target(self, target_name):
        self.target_name = target_name
        self.target = Target(target_name)
        self.project.target = self.target
        return self

    def with_cflags(self, cflags):
        self.project.target.cflags = cflags
        return self

    def with_cppflags(self, cppflags):
        self.project.target.cppflags = cppflags
        return self

    def with_ldflags(self, ldflags):
        self.project.target.ldflags = ldflags
        return self

    def with_libraries(self, libraries):
        self.project.target.libraries = libraries
        return self

    def with_toolchain_prefix(self, toolchain_prefix):
        self.project.target.toolchain_prefix = toolchain_prefix
        return self

    def with_package(self, path, sources, cflags):
        package = Package(path)
        package.sources = sources
        package.cflags = cflags
        self.project.target.packages.append(package)
        self.project.target.include_directories.add(path)
        return self

    def with_test(self, test_name):
        test = TestSuite(test_name, f'{test_name}.cpp')
        self.project.test.test_suites.append(test)
        return self

    def with_test_package(self, path, sources, cflags):
        package = Package(path)
        package.sources = sources
        package.cflags = cflags
        self.project.test.packages.append(package)
        self.project.test.include_directories.add(path)
        return self

    def with_test_ldflags(self, ldflags):
        self.project.test.ldflags = ldflags
        return self

    def sort_include_directories(self):
        self.target.include_directories = set(sorted(list(self.target.include_directories)))
        self.project.test.include_directories = set(sorted(list(self.project.test.include_directories)))
        for test in self.project.test.test_suites:
            test.include_directories = set(sorted(list(test.include_directories)))
        return self

    def with_bits(self, bits):
        self.project.target.bits.extend(bits)
        return self

    def with_test_bits(self, bits):
        self.project.test.bits.extend(bits)
        return self



from cpm.infrastructure import yaml_parser
from cpm.infrastructure import filesystem

GLOBAL_CONFIGURATION_FILENAME = '.cpm.yaml'
DEFAULT_CONFIGURATION = {
    'cpm_hub_url': 'https://repo.cpmbits.com:8000',
}


class CpmUserConfiguration(object):
    def __init__(self):
        self.global_configuration_file = f'{filesystem.home_directory()}/{GLOBAL_CONFIGURATION_FILENAME}'
        self.configuration = DEFAULT_CONFIGURATION.copy()

    def load(self):
        if filesystem.file_exists(self.global_configuration_file):
            configuration = yaml_parser.load(self.global_configuration_file)
            self.configuration.update(configuration)

    def __getitem__(self, item):
        return self.configuration.get(item, '')



import unittest
import mock
import shutil

from cpm.infrastructure.yaml_parser import YamlParser
from cpm.domain.constants import PROJECT_DESCRIPTOR_FILE
from cpm.domain.project import project_composer
from cpm.domain.project import project_descriptor_parser
from cpm.domain.project.project_descriptor import ProjectDescriptor, TargetDescription, CompilationPlan, PackageDescription


class TestProjectComposer(unittest.TestCase):
    def setUp(self):
        self.parser = YamlParser(pure=True)
        shutil.rmtree(PROJECT_DESCRIPTOR_FILE, ignore_errors=True)

    def tearDown(self):
        shutil.rmtree(PROJECT_DESCRIPTOR_FILE, ignore_errors=True)

    def test_should_compose_project_from_project_description_without_packages(self):
        self.write_descriptor({
            'name': 'HalfLife3',
            'version': '1.0',
            'description': 'I want to believe'
        })

        project_description = project_descriptor_parser.parse_from('.')
        project = project_composer.compose(project_description, 'default')

        assert project.name == 'HalfLife3'
        assert project.version == '1.0'
        assert project.description == 'I want to believe'
        assert project.target.name == 'default'
        assert project.target.main == 'main.cpp'

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_should_compose_project_from_project_description_with_one_build_package(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
            'build': {
                'packages': {
                    'shaders': {
                        'cflags': ['-DUSE_PORTAL_GUN']
                    }
                },
                'cflags': ['-Wall'],
                'ldflags': ['-pg'],
                'libraries': ['pthread'],
                'includes': ['./include']
            }
        })
        filesystem.find.side_effect = [['shaders/shader.cpp'], ['shaders/water.c'], []]
        filesystem.parent_directory.return_value = '.'

        project_description = project_descriptor_parser.parse_from('.')
        project = project_composer.compose(project_description, 'default')

        assert len(project.target.packages) == 1
        assert project.target.packages[0].path == 'shaders'
        assert project.target.packages[0].sources == ['shaders/shader.cpp', 'shaders/water.c']
        assert project.target.packages[0].cflags == ['-DUSE_PORTAL_GUN', '-Wall']
        assert project.target.cflags == ['-Wall']
        assert project.target.ldflags == ['-pg']
        assert project.target.libraries == ['pthread']
        assert project.target.include_directories == {'.', './include'}

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_should_raise_an_error_when_target_is_not_described(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
            'build': {
                'packages': {
                    'shaders': {
                        'cflags': ['-DUSE_PORTAL_GUN']
                    }
                },
            }
        })

        project_description = project_descriptor_parser.parse_from('.')
        self.assertRaises(project_composer.TargetNotDescribed, project_composer.compose, project_description, 'non-described-target')

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_should_compose_project_from_project_description_with_one_target_build_package(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
            'targets': {
                'default': {
                    'build': {
                        'packages': {
                            'shaders': {
                                'cflags': ['-DUSE_PORTAL_GUN']
                            }
                        },
                        'cflags': ['-Wall'],
                        'ldflags': ['-pg'],
                        'libraries': ['pthread'],
                        'includes': ['./include']
                    },
                    'post_build': ['./post_build.sh'],
                    'main': 'main.c',
                    'image': 'cpmbits/docker',
                    'test_image': 'cpmbits/docker_test',
                    'test_dockerfile': 'test.Dockerfile',
                    'toolchain_prefix': 'arm-linux-gnueabi-'
                }
            }
        })
        filesystem.find.side_effect = [['shaders/shader.cpp'], ['shaders/water.c'], []]
        filesystem.parent_directory.return_value = '.'

        project_description = project_descriptor_parser.parse_from('.')
        project = project_composer.compose(project_description, 'default')

        assert len(project.target.packages) == 1
        assert project.target.packages[0].path == 'shaders'
        assert project.target.packages[0].sources == ['shaders/shader.cpp', 'shaders/water.c']
        assert project.target.packages[0].cflags == ['-DUSE_PORTAL_GUN', '-Wall']
        assert project.target.cflags == ['-Wall']
        assert project.target.ldflags == ['-pg']
        assert project.target.libraries == ['pthread']
        assert project.target.include_directories == {'.', './include'}
        assert project.target.post_build == ['./post_build.sh']
        assert project.target.main == 'main.c'
        assert project.target.image == 'cpmbits/docker'
        assert project.target.test_image == 'cpmbits/docker_test'
        assert project.target.test_dockerfile == 'test.Dockerfile'
        assert project.target.toolchain_prefix == 'arm-linux-gnueabi-'

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_should_compose_project_from_project_description_with_one_target_test_package(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
            'test': {
                'includes': ['./test/include']
            },
            'targets': {
                'default': {
                    'test': {
                        'packages': {
                            'shaders': {
                                'cflags': ['-DUSE_PORTAL_GUN']
                            }
                        },
                        'cflags': ['-Wall'],
                        'ldflags': ['-pg'],
                        'libraries': ['pthread']
                    },
                }
            }
        })
        filesystem.find.side_effect = [['shaders/shader.cpp'], ['shaders/water.c'], []]
        filesystem.parent_directory.return_value = '.'

        project_description = project_descriptor_parser.parse_from('.')
        project = project_composer.compose(project_description, 'default')

        assert len(project.test.packages) == 1
        assert project.test.packages[0].path == 'shaders'
        assert project.test.packages[0].sources == ['shaders/shader.cpp', 'shaders/water.c']
        assert project.test.packages[0].cflags == ['-DUSE_PORTAL_GUN', '-Wall']
        assert project.test.cflags == ['-Wall']
        assert project.test.ldflags == ['-pg']
        assert project.test.libraries == ['pthread']
        assert project.test.include_directories == {'.', './test/include'}

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_should_compose_project_from_project_description_with_one_test(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
        })
        filesystem.find.side_effect = [['tests/test_one.cpp']]
        filesystem.parent_directory.return_value = '.'

        project_description = project_descriptor_parser.parse_from('.')
        project = project_composer.compose(project_description, 'default')

        assert len(project.test.test_suites) == 1
        assert project.test.test_suites[0].name == 'test_one'
        assert project.test.test_suites[0].main == 'tests/test_one.cpp'

    @mock.patch('cpm.domain.project.project_composer.filesystem')
    def test_compose_from_description_with_customized_bit_compilation(self, filesystem):
        self.write_descriptor({
            'name': 'HalfLife3',
            'build': {
                'bits': {
                    'arduino': {
                        'version': '1.0.0',
                        'target': 'nano33',
                        'cflags': ['-DBIT_FLAG']
                    }
                }
            }
        })
        filesystem.find.side_effect = [['tests/test_one.cpp'], ['nano33/nano33.cpp'], []]
        filesystem.parent_directory.return_value = '.'

        project_description = project_descriptor_parser.parse_from('.')
        arduino_bit = ProjectDescriptor(
            name='arduino',
            version='1.0.0',
            targets={
                'nano33': TargetDescription(
                    name='nano33',
                    build=CompilationPlan(
                        cflags=['-mcpu=atmel'],
                        packages=[PackageDescription(path='nano33')]
                    )
                )
            },
            declared_bit=project_description.build.declared_bits[0]
        )
        project_description.build.bits['arduino'] = arduino_bit
        project = project_composer.compose(project_description, 'default')

        assert project.target.bits[0].packages[0].path == 'bits/arduino/1.0.0/nano33'
        assert project.target.bits[0].packages[0].cflags == ['-mcpu=atmel', '-DBIT_FLAG']

    def write_descriptor(self, data):
        with open(PROJECT_DESCRIPTOR_FILE, 'w') as stream:
            self.parser.dump(data, stream)



import unittest
from unittest import mock

from cpm.domain.project_packager import PackagingFailure
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.infrastructure.http_client import HttpConnectionError
from cpm.api.publish import publish_project
from cpm.infrastructure.cpm_hub_connector_v1 import AuthenticationFailure
from cpm.infrastructure.cpm_hub_connector_v1 import InvalidCpmHubUrl
from cpm.infrastructure.cpm_hub_connector_v1 import PublicationFailure


class TestApiPublish(unittest.TestCase):
    def test_publish_fails_when_current_directory_is_not_a_cpm_project(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = ProjectDescriptorNotFound

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_build_fails_when_descriptor_contains_errors(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = ParseError('project.yaml', 1, 1, 'error')

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_project_contains_no_packages(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = PackagingFailure

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_connection_to_server_fails(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = HttpConnectionError

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_invalid_url_is_specified(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = InvalidCpmHubUrl

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_authentication_fails(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = AuthenticationFailure

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_publication_fails(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = PublicationFailure

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_fails_when_user_interrupts_with_ctrl_c(self):
        publish_service = mock.MagicMock()
        publish_service.publish.side_effect = KeyboardInterrupt

        result = publish_project(publish_service)

        assert result.status_code == 1
        publish_service.publish.assert_called_once()

    def test_publish_api(self):
        publish_service = mock.MagicMock()

        result = publish_project(publish_service)

        assert result.status_code == 0



import unittest
from mock import MagicMock
from mock import patch

import os

from cpm.domain.compilation_service import CompilationService
from cpm.domain.project_commands import DockerImageNotFound
from cpm.domain.project.project import Project, Target
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError


class TestCompilationService(unittest.TestCase):
    def setUp(self):
        self.cmakelists_builder = MagicMock()
        self.project_loader = MagicMock()
        self.project_commands = MagicMock()
        self.compilation_service = CompilationService(self.project_loader, self.cmakelists_builder, self.project_commands)

    def test_compilation_service_fails_when_project_loader_fails_to_load_project(self):
        self.project_loader.load.side_effect = ProjectDescriptorNotFound

        self.assertRaises(ProjectDescriptorNotFound, self.compilation_service.build)
        self.assertRaises(ProjectDescriptorNotFound, self.compilation_service.update)
        self.project_loader.load.assert_called()

    def test_compilation_service_generates_compilation_recipe_from_project_sources_and_compiles_project(self):
        project = Project('ProjectName')
        self.project_loader.load.return_value = project

        self.compilation_service.build('target_name')

        self.project_loader.load.assert_called_once_with('.', 'target_name')
        self.cmakelists_builder.build.assert_called_once_with(project)
        self.project_commands.build.assert_called_once_with(project)

    def test_compilation_service_only_generates_compilation_recipe_when_updating(self):
        project = Project('ProjectName')
        self.project_loader.load.return_value = project

        self.compilation_service.update('target_name')

        self.project_loader.load.assert_called_once_with('.', 'target_name')
        self.cmakelists_builder.build.assert_called_once_with(project)

    def test_clean_fails_when_project_loader_fails_to_load_project(self):
        self.project_loader.load.side_effect = ProjectDescriptorNotFound
        self.assertRaises(ProjectDescriptorNotFound, self.compilation_service.clean)
        self.project_loader.load.assert_called_once()

    def test_clean_uses_project_commands_to_clean_project(self):
        self.compilation_service.clean()

        self.project_loader.load.assert_called_once()
        self.project_commands.clean.assert_called_once()

    def test_clean_ignores_project_descriptor_parsing_errors(self):
        self.project_loader.load.side_effect = ParseError('project.yaml', 1, 1, 'error')

        self.compilation_service.clean()

        self.project_loader.load.assert_called_once()
        self.project_commands.clean.assert_called_once()



import unittest
from unittest import mock
from pathlib import Path

from cpm.infrastructure.yaml_parser import YamlParser

from cpm.domain.constants import PROJECT_DESCRIPTOR_FILE
from cpm.domain.project import project_descriptor_parser
from cpm.domain.project import project_descriptor_editor


class TestProjectDescriptorEditor(unittest.TestCase):
    @mock.patch('cpm.domain.project.project_descriptor_editor.filesystem')
    def test_parse_project_descriptor_with_just_the_project_information(self, filesystem):
        yaml = YamlParser()
        with open(PROJECT_DESCRIPTOR_FILE, 'w') as stream:
            stream.write(f'''name: 'bender bender rodriguez'
version: '1.0'
description: 'kill all humans'
build:
    packages:
    bits:
test:
targets:
''')
        yaml_document = yaml.load_from(Path(PROJECT_DESCRIPTOR_FILE))
        project_descriptor = project_descriptor_parser.digest_yaml(yaml_document)
        project_descriptor.yaml_document = yaml_document

        project_descriptor_editor.update('.', project_descriptor, {'name': 'pepito', 'version': '1.0.0'})

        filesystem.write_file.assert_called_with(f'./{PROJECT_DESCRIPTOR_FILE}', f'''name: pepito
version: 1.0.0
description: kill all humans
build:
  packages:
  bits:
test:
targets:
''')




import unittest
import mock

from cpm.domain.project.project import Project
from cpm.domain.bit_download import BitDownload
from cpm.domain.bit_installer import BitInstaller


class TestBitInstaller(unittest.TestCase):
    @mock.patch('cpm.domain.bit_installer.filesystem')
    def test_bit_installation_when_bit_was_not_installed_before(self, filesystem):
        project_loader = mock.MagicMock()
        installer = BitInstaller(project_loader)
        bit_download = BitDownload("cest", "1.0", "Yml0IHBheWxvYWQ=")
        project_loader.load.return_value = Project("cest")
        filesystem.directory_exists.return_value = False

        installer.install(bit_download)

        filesystem.create_directory.assert_called_once_with('bits/cest/1.0')
        filesystem.unzips.assert_called_once_with(b'bit payload', 'bits/cest/1.0')

    @mock.patch('cpm.domain.bit_installer.filesystem')
    def test_bit_installation_when_bit_was_installed_before(self, filesystem):
        project_loader = mock.MagicMock()
        installer = BitInstaller(project_loader)
        bit_download = BitDownload("cest", "1.0", "Yml0IHBheWxvYWQ=")
        project_loader.load.return_value = Project("cest")
        filesystem.directory_exists.return_value = True

        installer.install(bit_download)

        filesystem.remove_directory.assert_called_once_with('bits/cest/1.0')
        filesystem.create_directory.assert_called_once_with('bits/cest/1.0')
        filesystem.unzips.assert_called_once_with(b'bit payload', 'bits/cest/1.0')



import unittest
import mock

from cpm.domain.constants import INITIAL_PROJECT_VERSION, PROJECT_DESCRIPTOR_FILE
from cpm.domain.project.project import Project
from cpm.domain.project.project_template import ProjectTemplate
from cpm.domain.sample_code import CPP_HELLO_WORLD
from cpm.domain.creation_service import CreationService
from cpm.domain.creation_service import CreationOptions
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound


class TestCreationService(unittest.TestCase):
    def test_it_instantiation(self):
        project_loader = mock.MagicMock()
        CreationService(project_loader)

    def test_project_exists_when_loading_the_project_succeeds(self):
        project_loader = mock.MagicMock()
        project_loader.load.return_value = Project('dummy')
        creation_service = CreationService(project_loader)
        directory = '.'

        assert creation_service.exists(directory)
        project_loader.load.assert_called_once_with(directory)

    def test_project_does_not_exist_when_loading_the_project_fails(self):
        project_loader = mock.MagicMock()
        project_loader.load.side_effect = ProjectDescriptorNotFound
        creation_service = CreationService(project_loader)
        directory = 'project_location'

        assert not creation_service.exists(directory)
        project_loader.load.assert_called_once_with(directory)

    @mock.patch('cpm.domain.creation_service.filesystem')
    def test_it_creates_directory_and_descriptor_file_when_creating_project(self, filesystem):
        project_loader = mock.MagicMock()
        creation_service = CreationService(project_loader)
        creation_options = CreationOptions(
            generate_sample_code=False,
            directory='AwesomeProject',
            project_name='AwesomeProject'
        )

        creation_service.create(creation_options)

        filesystem.create_directory.assert_has_calls([
            mock.call('AwesomeProject'),
            mock.call('AwesomeProject/tests')
        ])
        filesystem.create_file.assert_called_once_with(
            'AwesomeProject/project.yaml',
            mock.ANY
        )

    @mock.patch('cpm.domain.creation_service.filesystem')
    def test_it_only_creates_descriptor_file_when_creating_project_from_existing_sources(self, filesystem):
        project_loader = mock.MagicMock()
        creation_service = CreationService(project_loader)
        creation_options = CreationOptions(
            generate_sample_code=False,
            directory='.',
            project_name='AwesomeProject',
            init_from_existing_sources=True
        )

        creation_service.create(creation_options)

        filesystem.create_directory.assert_not_called()
        filesystem.create_file.assert_called_once_with(
            './project.yaml',
            mock.ANY
        )

    @mock.patch('cpm.domain.creation_service.filesystem')
    def test_creation_service_generates_default_sample_code_when_selected(self, filesystem):
        project_loader = mock.MagicMock()
        creation_service = CreationService(project_loader)
        creation_options = CreationOptions(
            directory='AwesomeProject',
            project_name='AwesomeProject'
        )

        creation_service.create(creation_options)

        filesystem.create_file.assert_called_with(
            'AwesomeProject/main.cpp',
            CPP_HELLO_WORLD
        )

    @mock.patch('cpm.domain.creation_service.filesystem')
    def test_creation_service_returns_generated_project(self, filesystem):
        project_loader = mock.MagicMock()
        creation_service = CreationService(project_loader)
        creation_options = CreationOptions(
            project_name='AwesomeProject'
        )

        project = creation_service.create(creation_options)

        assert project.name == 'AwesomeProject'

    @mock.patch('cpm.domain.creation_service.project_descriptor_editor')
    def test_creation_service_uses_cpm_hub_connector_to_download_template(self, project_descriptor_editor):
        project_loader = mock.MagicMock()
        cpm_hub_connector = mock.MagicMock()
        template_installer = mock.MagicMock()
        creation_service = CreationService(project_loader, cpm_hub_connector, template_installer)
        options = CreationOptions(
            project_name='AwesomeProject',
            init_from_template=True,
            template_name='arduino-uno',
        )
        template_project = Project(name=options.project_name, version=INITIAL_PROJECT_VERSION)
        project_loader.load.return_value = template_project
        project_template = ProjectTemplate(
            name='arduino-uno',
            version='1.0.0',
            payload='payload'
        )
        cpm_hub_connector.download_template.return_value = project_template

        project = creation_service.create(options)

        assert project.name == 'AwesomeProject'
        template_installer.install.assert_called_once_with(project_template, options.directory)
        project_descriptor_editor.update.assert_called_once()




import unittest
import mock

from cpm.api.prep import prep_project
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError


class TestApiUpdate(unittest.TestCase):
    def test_update_fails_when_current_directory_is_not_a_cpm_project(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.update.side_effect = ProjectDescriptorNotFound()

        result = prep_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.update.assert_called_once()

    def test_build_fails_when_descriptor_contains_errors(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()
        compilation_service.update.side_effect = ParseError('project.yaml', 1, 1, 'error')

        result = prep_project(compilation_service, recipe)

        assert result.status_code == 1
        compilation_service.update.assert_called_once()

    def test_update_project(self):
        recipe = mock.MagicMock()
        compilation_service = mock.MagicMock()

        result = prep_project(compilation_service, recipe)

        assert result.status_code == 0
        compilation_service.update.assert_called_once()



from cpm.argument_parser import ArgumentParser
from cpm.api.result import Result
from cpm.api.result import OK
from cpm.api.result import FAIL
from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
from cpm.domain.project_commands import ProjectCommands
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound
from cpm.domain.project.project_loader import ProjectLoader
from cpm.domain.compilation_service import CompilationService


def clean_project(compilation_service):
    try:
        compilation_service.clean()
    except ProjectDescriptorNotFound:
        return Result(FAIL, f'error: not a cpm project')

    return Result(OK, f'clean finished')


def execute(argv):
    project_loader = ProjectLoader()
    cmakelists_builder = CMakeListsBuilder()
    project_builder = ProjectCommands()
    service = CompilationService(project_loader, cmakelists_builder, project_builder)

    result = clean_project(service)

    return result


def argument_parser():
    return ArgumentParser(prog='cpm clean', description=description())


def print_help():
    return argument_parser().print_help()


def description():
    return 'clean all build files'



from cpm.domain.project_packager import ProjectPackager
from cpm.domain.template_packager import TemplatePackager
from cpm.domain.project import project_descriptor_parser


class PublishService(object):
    def __init__(self, cpm_hub_connector, project_packager=ProjectPackager(), template_packager=TemplatePackager()):
        self.template_packager = template_packager
        self.project_packager = project_packager
        self.cpm_hub_connector = cpm_hub_connector

    def publish(self):
        project_descriptor = project_descriptor_parser.parse_from('.')
        package_name = self.project_packager.pack(project_descriptor, 'dist')
        self.cpm_hub_connector.publish_bit(project_descriptor, package_name)

    def publish_template(self):
        project_descriptor = project_descriptor_parser.parse_from('.')
        package_name = self.template_packager.pack(project_descriptor, 'dist')
        self.cpm_hub_connector.publish_template(project_descriptor, package_name)



import unittest
import mock

from cpm.domain.project_packager import ProjectPackager
from cpm.domain.project_packager import PackagingFailure
from cpm.domain.project.project_descriptor import ProjectDescriptor, TargetDescription, PackageDescription


class TestBitPackager(unittest.TestCase):
    def test_packaging_bit_raises_exception_when_no_packages_are_declared(self):
        packager = ProjectPackager()
        project = ProjectDescriptor('cest')

        self.assertRaises(PackagingFailure, packager.pack, project, 'dist')

    @mock.patch('cpm.domain.project_packager.filesystem')
    def test_packaging_bit_raises_exception_when_output_directory_exists(self, filesystem):
        filesystem.directory_exists.return_value = True
        packager = ProjectPackager()
        project = ProjectDescriptor('cest')
        project.build.packages.append(PackageDescription('fakeit'))

        self.assertRaises(PackagingFailure, packager.pack, project, 'dist')

    @mock.patch('cpm.domain.project_packager.filesystem')
    def test_packaging_creates_output_directory(self, filesystem):
        filesystem.directory_exists.return_value = False
        packager = ProjectPackager()
        project = ProjectDescriptor('cest')
        project.targets['default'] = TargetDescription('default')
        project.build.packages.append(PackageDescription('api'))
        project.targets['default'].build.packages.append(PackageDescription('domain'))

        packager.pack(project, 'dist')

        filesystem.create_directory.assert_called_once_with('dist')
        filesystem.copy_file.assert_called_once_with('project.yaml', 'dist/project.yaml')
        filesystem.copy_directory.assert_has_calls([
            mock.call('api', 'dist/api'),
            mock.call('domain', 'dist/domain'),
        ])
        filesystem.zip.assert_called_once_with('dist', 'cest')
        filesystem.remove_directory.assert_called_once_with('dist')






from dataclasses import dataclass, field


@dataclass
class DeclaredBit:
    name: str
    version: str
    target: str = ''
    cflags: list = field(default_factory=list)
    cppflags: list = field(default_factory=list)


@dataclass
class PackageDescription:
    path: str
    cflags: list = field(default_factory=list)
    cppflags: list = field(default_factory=list)
    sources: list = field(default_factory=list)


@dataclass
class CompilationPlan:
    declared_bits: list = field(default_factory=list)
    bits: dict = field(default_factory=dict)
    packages: list = field(default_factory=list)
    cflags: list = field(default_factory=list)
    cppflags: list = field(default_factory=list)
    ldflags: list = field(default_factory=list)
    libraries: list = field(default_factory=list)
    includes: set = field(default_factory=set)


@dataclass
class ProjectInformation:
    name: str = ''
    version: str = ''
    description: str = ''


@dataclass
class TargetDescription:
    name: str
    main: str = 'main.cpp'
    format: str = 'binary'
    build: CompilationPlan = field(default_factory=CompilationPlan)
    test: CompilationPlan = field(default_factory=CompilationPlan)
    post_build: list = field(default_factory=list)
    dockerfile: str = ''
    image: str = ''
    test_image: str = ''
    test_dockerfile: str = ''
    toolchain_prefix: str = ''


@dataclass
class ProjectDescriptor:
    name: str = ''
    version: str = ''
    description: str = ''
    schema: str = '1.0'
    parser: object = None
    yaml_document: object = None
    build: CompilationPlan = field(default_factory=CompilationPlan)
    test: CompilationPlan = field(default_factory=CompilationPlan)
    declared_bit: DeclaredBit = None
    targets: dict = field(default_factory=dict)

    def build_packages(self):
        return self.build.packages + [package for target, description in self.targets.items() for package in description.build.packages]

    def included_files(self):
        return [] if self.parser is None else self.parser.included_files()



import unittest
import mock

from cpm.domain.template_packager import TemplatePackager
from cpm.domain.template_packager import FailedToPackageTemplate
from cpm.domain.project.project_descriptor import ProjectDescriptor, TargetDescription, PackageDescription


class TestTemplatePackager(unittest.TestCase):
    @mock.patch('cpm.domain.template_packager.filesystem')
    def test_packager_raises_exception_when_output_directory_exists(self, filesystem):
        filesystem.directory_exists.return_value = True
        packager = TemplatePackager()
        project = ProjectDescriptor('cest')
        project.build.packages.append(PackageDescription('fakeit'))

        self.assertRaises(FailedToPackageTemplate, packager.pack, project, 'dist')

    @mock.patch('cpm.domain.template_packager.filesystem')
    def test_packager_copies_all_to_output_directory_then_zips(self, filesystem):
        packager = TemplatePackager()
        project = ProjectDescriptor('cest')
        project.targets['default'] = TargetDescription('default')
        project.build.packages.append(PackageDescription('api'))
        project.test.packages.append(PackageDescription('test'))
        project.targets['default'].build.packages.append(PackageDescription('domain'))
        project.targets['default'].dockerfile = 'Dockerfile'
        filesystem.directory_exists.return_value = False
        filesystem.path_to.return_value = ''

        packager.pack(project, 'dist')

        filesystem.create_directory.assert_called_once_with('dist')
        filesystem.copy_file.assert_has_calls([
            mock.call('project.yaml', 'dist/project.yaml'),
            mock.call('main.cpp', 'dist/'),
            mock.call('Dockerfile', 'dist/')
        ])
        filesystem.copy_directory.assert_has_calls([
            mock.call('api', 'dist/api'),
            mock.call('domain', 'dist/domain'),
            mock.call('test', 'dist/test')
        ])
        filesystem.zip.assert_called_once_with('dist', 'cest')
        filesystem.remove_directory.assert_called_once_with('dist')






import base64

from cpm.infrastructure import filesystem


class TemplateInstaller:
    def install(self, template_download, directory):
        if filesystem.directory_exists(directory):
            raise UnableToInstallTemplate
        filesystem.create_directory(directory)
        filesystem.unzips(base64.b64decode(template_download.payload), directory)


class UnableToInstallTemplate(RuntimeError):
    pass



import unittest
import mock

from cpm.api.test import run_tests
from cpm.domain.project_commands import BuildError
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound, ParseError
from cpm.domain.project_commands import TestsFailed
from cpm.domain.test_service import NoTestsFound


class TestApiTest(unittest.TestCase):
    def test_run_tests_fails_when_current_directory_is_not_a_cpm_project(self):
        test_service = mock.MagicMock()
        test_service.run_tests.side_effect = ProjectDescriptorNotFound()

        result = run_tests(test_service)

        assert result.status_code == 1
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_build_fails_when_descriptor_contains_errors(self):
        test_service = mock.MagicMock()
        test_service.run_tests.side_effect = ParseError('project.yaml', 1, 1, 'error')

        result = run_tests(test_service)

        assert result.status_code == 1
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_run_tests_fails_when_no_tests_are_found(self):
        test_service = mock.MagicMock()
        test_service.run_tests.side_effect = NoTestsFound()

        result = run_tests(test_service)

        assert result.status_code == 0
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_run_tests_fails_when_compilation_fails(self):
        test_service = mock.MagicMock()
        test_service.run_tests.side_effect = BuildError()

        result = run_tests(test_service)

        assert result.status_code == 1
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_run_tests_fails_when_tests_fail(self):
        test_service = mock.MagicMock()
        test_service.run_tests.side_effect = TestsFailed()

        result = run_tests(test_service)

        assert result.status_code == 1
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_run_project_tests(self):
        test_service = mock.MagicMock()

        result = run_tests(test_service)

        assert result.status_code == 0
        test_service.run_tests.assert_called_once_with((), 'default', ())

    def test_run_project_tests_with_patterns(self):
        test_service = mock.MagicMock()

        result = run_tests(test_service, files_or_dirs=['tests'])

        assert result.status_code == 0
        test_service.run_tests.assert_called_once_with(['tests'], 'default', ())

    def test_run_project_tests_with_target_other_than_default(self):
        test_service = mock.MagicMock()

        result = run_tests(test_service, target='rpi4')

        assert result.status_code == 0
        test_service.run_tests.assert_called_once_with((), 'rpi4', ())

    def test_run_project_tests_with_test_args(self):
        test_service = mock.MagicMock()

        result = run_tests(test_service, test_args=['arg1'])

        assert result.status_code == 0
        test_service.run_tests.assert_called_once_with((), 'default', ['arg1'])



import unittest
import mock

from cpm.domain.project import project_loader
from cpm.domain.project.project_descriptor import ProjectDescriptor, DeclaredBit
from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound


class TestProjectLoader(unittest.TestCase):
    @mock.patch('cpm.domain.project.project_loader.project_descriptor_parser')
    @mock.patch('cpm.domain.project.project_loader.project_composer')
    def test_project_loader_without_bits(self, project_composer, project_descriptor_parser):
        # Given
        project_descriptor_parser.parse_from.return_value = ProjectDescriptor()
        loader = project_loader.ProjectLoader()
        # When
        loader.load('.', 'default')
        # Then
        project_descriptor_parser.parse_from.assert_called_once_with('.')
        project_composer.compose.assert_called_once_with(project_descriptor_parser.parse_from.return_value, 'default')

    @mock.patch('cpm.domain.project.project_loader.project_descriptor_parser')
    @mock.patch('cpm.domain.project.project_loader.project_composer')
    def test_project_loader_with_declared_bits(self, project_composer, project_descriptor_parser):
        # Given
        project_descriptor = ProjectDescriptor()
        project_descriptor.build.declared_bits = [DeclaredBit('bit', '2.2')]
        project_descriptor_parser.parse_from.return_value = project_descriptor
        loader = project_loader.ProjectLoader()
        # When
        loader.load('.', 'default')
        # Then
        project_descriptor_parser.parse_from.assert_has_calls([
            mock.call('.'),
            mock.call('bits/bit/2.2')
        ])
        project_composer.compose.assert_called_once_with(project_descriptor, 'default')

    @mock.patch('cpm.domain.project.project_loader.project_descriptor_parser')
    @mock.patch('cpm.domain.project.project_loader.project_composer')
    def test_loading_project_when_some_bit_is_not_installed(self, project_composer, project_descriptor_parser):
        # Given
        project_descriptor = ProjectDescriptor()
        project_descriptor.build.declared_bits = [DeclaredBit('bit', '2.2')]
        project_descriptor_parser.parse_from.side_effect = [project_descriptor, ProjectDescriptorNotFound]
        loader = project_loader.ProjectLoader()
        # When
        loader.load('.', 'default')
        # Then
        project_descriptor_parser.parse_from.assert_has_calls([
            mock.call('.'),
            mock.call('bits/bit/2.2')
        ])
        project_composer.compose.assert_called_once_with(project_descriptor, 'default')




import unittest

from cpm.domain.project import project_descriptor
from cpm.domain.project import project_descriptor_parser


class TestProjectDescriptorParser(unittest.TestCase):
    def test_parse_project_descriptor_with_just_the_project_information(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'build': {
                'packages': None,
                'bits': None,
                'cflags': None
            },
            'test': None,
            'targets': None
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project == project_descriptor.ProjectDescriptor(
            name='bender bender rodriguez',
            version='1.0',
            description='kill all humans',
            targets={'default': project_descriptor.TargetDescription('default')})

    def test_parse_project_descriptor_with_build_compilation_plan(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'build': {
                'packages': {
                    'cpmhub/bits': {
                        'cflags': ['-DHOLA'],
                        'cppflags': ['-DHOLA_CPP'],
                    },
                    'cpmhub/http': None
                },
                'bits': {
                    'sqlite3': '3.32.3'
                },
                'cflags': ['-std=c99'],
                'cppflags': ['-std=c++11'],
                'ldflags': ['-pg'],
                'libraries': ['pthread'],
                'includes': ['./include']
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.build.packages == [
            project_descriptor.PackageDescription('cpmhub/bits', cflags=['-DHOLA'], cppflags=['-DHOLA_CPP']),
            project_descriptor.PackageDescription('cpmhub/http')
        ]
        assert project.build.cflags == ['-std=c99']
        assert project.build.cppflags == ['-std=c++11']
        assert project.build.ldflags == ['-pg']
        assert project.build.libraries == ['pthread']
        assert project.build.includes == {'./include'}
        assert len(project.build.declared_bits) == 1
        assert project.build.declared_bits[0] == project_descriptor.DeclaredBit('sqlite3', '3.32.3')

    def test_parse_project_descriptor_with_test_compilation_plan(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'test': {
                'packages': {
                    'cpmhub/bits': {},
                    'cpmhub/http': {}
                }
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.test.packages == [
            project_descriptor.PackageDescription('cpmhub/bits'),
            project_descriptor.PackageDescription('cpmhub/http')
        ]

    def test_parse_project_descriptor_with_default_target_image(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'targets': {
                'default': {
                    'image': 'cpmbits/bender',
                    'main': 'main.c',
                    'test_image': 'cpmbits/bender_test',
                    'test_dockerfile': 'test.Dockerfile',
                    'toolchain_prefix': 'arm-linux-gnueabi-'
                }
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.targets == {
            'default': project_descriptor.TargetDescription(
                'default',
                image='cpmbits/bender',
                main='main.c',
                test_image='cpmbits/bender_test',
                test_dockerfile='test.Dockerfile',
                toolchain_prefix='arm-linux-gnueabi-'
            )
        }

    def test_parse_project_descriptor_with_target_build_compilation_plan(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'targets': {
                'arduino': {
                    'dockerfile': 'Dockerfile',
                    'build': {
                        'packages': {
                            'arduino': {},
                        },
                        'ldflags': ['-Wl,--wrap=malloc']
                    },
                    'post_build': [
                        './scripts/package.sh'
                    ]
                }
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.targets['arduino'].build.packages == [
            project_descriptor.PackageDescription('arduino')
        ]
        assert project.targets['arduino'].build.ldflags == ['-Wl,--wrap=malloc']
        assert project.build_packages() == [
            project_descriptor.PackageDescription('arduino')
        ]
        assert project.targets['arduino'].dockerfile == 'Dockerfile'
        assert project.targets['arduino'].post_build == ['./scripts/package.sh']

    def test_parse_project_descriptor_with_target_test_compilation_plan(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'targets': {
                'arduino': {
                    'image': 'cpmbits/arduino',
                    'test': {
                        'packages': {
                            'arduino': {},
                        }
                    }
                }
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.targets['arduino'].test.packages == [
            project_descriptor.PackageDescription('arduino')
        ]

    def test_parse_project_descriptor_with_customized_bit_compilation(self):
        yaml_contents = {
            'name': 'bender bender rodriguez',
            'version': '1.0',
            'description': 'kill all humans',
            'build': {
                'packages': {
                    'cpmhub/http': None
                },
                'bits': {
                    'sqlite3': {
                        'version': '3.32.3',
                        'cflags': ['-DCUSTOM_BIT_DEFINE'],
                        'target': 'arduinoNano33'
                    }
                }
            }
        }
        project = project_descriptor_parser.digest_yaml(yaml_contents)
        assert project.build.packages == [
            project_descriptor.PackageDescription('cpmhub/http')
        ]
        assert project.build.declared_bits[0] == project_descriptor.DeclaredBit(
            name='sqlite3', version='3.32.3', cflags=['-DCUSTOM_BIT_DEFINE'], target='arduinoNano33'
        )



import unittest
import mock

from cpm.api.create import new_project
from cpm.domain.creation_service import CreationOptions
from cpm.infrastructure.cpm_hub_connector_v1 import TemplateNotFound


class TestApiCreate(unittest.TestCase):
    def test_create_project_uses_creation_service_for_initializing_project(self):
        creation_service = mock.MagicMock()
        creation_service.exists.return_value = False
        options = CreationOptions(
            project_name='Awesome Project'
        )

        result = new_project(creation_service, options)

        assert result.status_code == 0
        creation_service.create.assert_called_once_with(options)

    def test_returns_error_code_when_project_already_exists(self):
        creation_service = mock.MagicMock()
        creation_service.exists.return_value = True
        options = CreationOptions(
            project_name='Awesome Project'
        )

        result = new_project(creation_service, options)

        assert result.status_code == 1

    def test_create_project_accepts_options_from_api(self):
        creation_service = mock.MagicMock()
        creation_service.exists.return_value = False
        options = CreationOptions(
            project_name='Awesome Project',
            generate_sample_code=True
        )

        result = new_project(creation_service, options)

        assert result.status_code == 0
        creation_service.create.assert_called_once_with(options)

    def test_create_project_fails_when_template_used_does_not_exist(self):
        creation_service = mock.MagicMock()
        creation_service.exists.return_value = False
        options = CreationOptions(
            project_name='Awesome Project'
        )
        creation_service.create.side_effect = TemplateNotFound

        result = new_project(creation_service, options)

        assert result.status_code == 1
        creation_service.create.assert_called_once()



import unittest
import mock
import os
import shutil
import zipfile

from cpm.domain.constants import PROJECT_DESCRIPTOR_FILE
from cpm.infrastructure import filesystem
from cpm.infrastructure import yaml_parser
from cpm.api import create
from cpm.api import install
from cpm.api import build
from cpm.api import publish
from cpm.api import test
from cpm.api.result import Result


class TestCpm(unittest.TestCase):
    PROJECT_NAME = 'test_project'
    PROJECT_NAME_FROM_TEMPLATE = 'templated_project'
    TEST_DIRECTORY = f'{os.path.dirname(os.path.abspath(__file__))}'
    PROJECT_DIRECTORY = f'{TEST_DIRECTORY}/{PROJECT_NAME}'
    PROJECT_FROM_TEMPLATE_DIRECTORY = f'{TEST_DIRECTORY}/{PROJECT_NAME_FROM_TEMPLATE}'

    def setUp(self):
        self.cwd = os.getcwd()
        shutil.rmtree(self.PROJECT_DIRECTORY, ignore_errors=True)
        shutil.rmtree(self.PROJECT_FROM_TEMPLATE_DIRECTORY, ignore_errors=True)
        os.chdir(self.TEST_DIRECTORY)
        create.execute([self.PROJECT_NAME])

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(self.PROJECT_DIRECTORY, ignore_errors=True)
        shutil.rmtree(self.PROJECT_FROM_TEMPLATE_DIRECTORY, ignore_errors=True)

    def test_build(self):
        os.chdir(self.PROJECT_DIRECTORY)
        result = build.execute([])
        assert result.status_code == 0

    def test_build_with_user_defined_include(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_includes(['../environment'])
        self.add_main_with_user_include()
        result = build.execute([])
        assert result.status_code == 0

    def test_bit_installation_from_command_line_passed_bit(self):
        os.chdir(self.PROJECT_DIRECTORY)
        result = install.execute(['-s', 'http://localhost:8000', 'test:1.0'])
        assert result == Result(0, 'installed bit test:1.0')

    def test_recursive_bit_installation(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('build', 'test', '1.0')
        result = install.execute(['-s', 'http://localhost:8000'])
        assert result == Result(0, 'installed bits')

    def test_build_after_recursive_bit_installation(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('build', 'test', '1.0')
        self.set_libraries(['pthread', 'dl'])
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_build_after_recursive_bit_installation_for_target_not_described_by_bit(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('build', 'test', '1.0')
        self.set_libraries(['pthread', 'dl'])
        self.set_target_main('rpi4', 'main.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute(['rpi4'])
        assert result == Result(0, 'Build finished')

    def test_build_when_declared_bit_is_not_installed(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('build', 'test', '1.0')
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_build_from_docker_image(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_build_from_docker_image_with_ldflags(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_ldflags(['-Wl,--allow-multiple-definition'])
        self.set_target_ldflags('default', ['-Wl,-s'])
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_build_from_docker_image_with_toolchain_prefix(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_dockerfile('default', f'../environment/Dockerfile')
        self.set_toolchain_prefix('default', 'arm-linux-gnueabihf-')
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_run_tests_from_docker_image(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_target_test_image('default', 'cpmbits/ubuntu:20.04')
        self.set_test_ldflags(['-Wl,-s'])
        self.add_test('test_case.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute([])
        assert result.status_code == 0

    def test_run_tests_using_dockerfile(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_target_test_dockerfile('default', f'../environment/Dockerfile')
        self.set_test_ldflags(['-Wl,-s'])
        self.add_test('test_case.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute([])
        assert result.status_code == 0

    def test_build_from_docker_image_with_non_default_target(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_image('ubuntu', 'cpmbits/ubuntu:20.04')
        self.set_target_main('ubuntu', 'main.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_build_from_docker_image_with_post_build(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_post_build('default', [f'ls', f'rm build/{self.PROJECT_NAME}'])
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')
        assert not filesystem.file_exists(f'build/{self.PROJECT_NAME}')

    def test_build_from_dockerfile(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_dockerfile('default', f'../environment/Dockerfile')
        install.execute(['-s', 'http://localhost:8000'])
        result = build.execute([])
        assert result == Result(0, 'Build finished')

    def test_after_recursive_bit_installation(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.set_test_cppflags(['-std=c++11'])
        self.add_test('test_case.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute([])
        assert result.status_code == 0

    def test_specifying_test_file(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.set_test_cppflags(['-std=c++11'])
        self.add_test('test_case1.cpp')
        self.add_test('test_case2.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute(['tests/test_case1.cpp'])
        assert result.status_code == 0

    def test_specifying_test_directory(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.set_test_cppflags(['-std=c++11'])
        self.add_test('test_case1.cpp')
        self.add_test('test_case2.cpp')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute(['tests'])
        assert result.status_code == 0

    def test_using_reference_to_external_file_in_cflags(self):
        os.chdir(self.PROJECT_DIRECTORY)
        project_yaml = '''build:
  bits:
  packages:
name: test_project
targets:
  default:
    main: main.cpp
test:
  bits:
    cest: '1.0'
  cflags: !include test_cflags.yaml
version: 0.0.1
'''
        with open('project.yaml', 'w') as stream:
            stream.write(project_yaml)
        yaml_parser.dump(['-std=c++11'], 'test_cflags.yaml')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute(['tests'])
        assert result.status_code == 0

    def test_failing_test_after_recursive_bit_installation(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.add_test('test_case.cpp', fails=True)
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute([])
        assert result.status_code == 1

    def test_failing_test_when_running_inside_docker_image(self):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.add_test('test_case.cpp', fails=True)
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_target_test_image('default', 'cpmbits/ubuntu:20.04')
        install.execute(['-s', 'http://localhost:8000'])
        result = test.execute([])
        assert result.status_code == 1

    @mock.patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    @mock.patch('builtins.input')
    def test_publishing_template(self, user_input, getpass):
        user_input.return_value = 'user'
        getpass.return_value = 'password'
        os.chdir(self.PROJECT_DIRECTORY)
        self.set_target_dockerfile('default', f'../environment/Dockerfile')
        install.execute(['-s', 'http://localhost:8000'])
        result = publish.execute(['-t', '-s', 'http://localhost:8000'])
        assert result == Result(0, 'Project published')

    @mock.patch('cpm.infrastructure.cpm_hub_connector_v1.getpass')
    @mock.patch('builtins.input')
    def test_creating_project_from_template(self, user_input, getpass):
        os.chdir(self.PROJECT_DIRECTORY)
        self.add_bit('test', 'cest', '1.0')
        self.add_test('test_case.cpp', fails=True)
        self.set_target_image('default', 'cpmbits/ubuntu:20.04')
        self.set_target_test_image('default', 'cpmbits/ubuntu:20.04')
        self.set_target_dockerfile('default', f'../environment/Dockerfile')
        user_input.return_value = 'user'
        getpass.return_value = 'password'
        publish.execute(['-t', '-s', 'http://localhost:8000'])
        os.chdir('..')
        shutil.rmtree(self.PROJECT_DIRECTORY, ignore_errors=True)
        result = create.execute(['-s', 'http://localhost:8000', '-t', f'{self.PROJECT_NAME}:0.0.1', self.PROJECT_NAME_FROM_TEMPLATE])
        assert result == Result(0, f'Created project {self.PROJECT_NAME_FROM_TEMPLATE}')

    def test_packaging_including_external_files(self):
        from cpm.domain.project_packager import ProjectPackager
        from cpm.domain.project import project_descriptor_parser
        os.chdir(self.PROJECT_DIRECTORY)
        project_yaml = '''build:
  bits:
  packages:
    empty_package:
name: test_project
targets:
  default:
    main: main.cpp
test:
  bits:
    cest: '1.0'
  cflags: !include test_cflags.yaml
version: 0.0.1
'''
        with open('project.yaml', 'w') as stream:
            stream.write(project_yaml)
        yaml_parser.dump(['-std=c++11'], 'test_cflags.yaml')
        os.mkdir('empty_package')
        project_descriptor = project_descriptor_parser.parse_from('.')
        pack = ProjectPackager().pack(project_descriptor, 'dist')
        zip = zipfile.ZipFile(pack)
        files_in_zip = zip.namelist()
        assert 'test_cflags.yaml' in files_in_zip

    def add_bit(self, plan, name, version):
        project_descriptor = yaml_parser.load(PROJECT_DESCRIPTOR_FILE)
        if plan not in project_descriptor or not project_descriptor[plan]:
            project_descriptor[plan] = {}
        project_descriptor[plan]['bits'] = {
            name: version
        }
        yaml_parser.dump(project_descriptor, PROJECT_DESCRIPTOR_FILE)

    def add_test(self, file_name, fails=False):
        if not filesystem.directory_exists('tests'):
            filesystem.create_directory('tests')
        expect = 'false' if fails else 'true'
        filesystem.create_file(
            f'tests/{file_name}',
            '#include <cest/cest.h>\n'
            'using namespace cest;\n'
            'describe("Test Case", []() {\n'
            '    it("passes", []() {\n'
            f'        expect(true).toBe({expect});\n'
            '    });\n'
            '});\n'
        )

    def set_target_image(self, target_name, image):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'image': image})
        )

    def set_toolchain_prefix(self, target_name, toolchain_prefix):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'toolchain_prefix': toolchain_prefix})
        )

    def set_target_test_image(self, target_name, image):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'test_image': image})
        )

    def set_target_main(self, target_name, main):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'main': main})
        )

    def set_post_build(self, target_name, post_build):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'post_build': post_build})
        )

    def set_target_dockerfile(self, target_name, dockerfile):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'dockerfile': dockerfile})
        )

    def set_target_test_dockerfile(self, target_name, dockerfile):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'test_dockerfile': dockerfile})
        )

    def set_target_ldflags(self, target_name, ldflags):
        self.modify_descriptor(
            lambda descriptor: descriptor.setdefault('targets', {}).setdefault(target_name, {}).update({'ldflags': ldflags})
        )

    def set_includes(self, includes):
        self.modify_descriptor(
            lambda descriptor: descriptor['build'].update({'includes': includes})
        )

    def set_ldflags(self, ldflags):
        self.modify_descriptor(
            lambda descriptor: descriptor['build'].update({'ldflags': ldflags})
        )

    def set_libraries(self, libraries):
        self.modify_descriptor(
            lambda descriptor: descriptor['build'].update({'libraries': libraries})
        )

    def set_test_cflags(self, cflags):
        self.modify_descriptor(
            lambda descriptor: descriptor['test'].update({'cflags': cflags})
        )

    def set_test_cppflags(self, cppflags):
        self.modify_descriptor(
            lambda descriptor: descriptor['test'].update({'cppflags': cppflags})
        )

    def set_test_ldflags(self, ldflags):
        self.modify_descriptor(
            lambda descriptor: descriptor['test'].update({'ldflags': ldflags})
        )

    def modify_descriptor(self, modify):
        project_descriptor = yaml_parser.load(PROJECT_DESCRIPTOR_FILE)
        modify(project_descriptor)
        yaml_parser.dump(project_descriptor, PROJECT_DESCRIPTOR_FILE)

    def add_main_with_user_include(self):
        filesystem.create_file(
            f'main.cpp',
            '#include <user_include.h>\n'
            'int main(int argc, char *argv[]) {\n'
            '    return 0;\n'
            '}\n'
        )



from cpm.domain.constants import DEFAULT_TARGET
from cpm.domain.constants import bit_directory
from cpm.infrastructure import filesystem
from cpm.domain.project.project_descriptor import TargetDescription
from cpm.domain.project.project import Project, Target, Package, TestSuite


def compose(project_descriptor, target_name):
    if target_name not in project_descriptor.targets:
        raise TargetNotDescribed()
    project = Project(
        project_descriptor.name,
        version=project_descriptor.version,
        description=project_descriptor.description,
        descriptor=project_descriptor
    )
    project.target = compose_target(target_name, project_descriptor)
    compose_tests(target_name, project_descriptor, project)
    return project


def compose_target(target_name, project_descriptor):
    target = Target(target_name)
    target_description = project_descriptor.targets.get(target_name, TargetDescription(target_name))
    target.cflags = project_descriptor.build.cflags + target_description.build.cflags
    target.cppflags = project_descriptor.build.cppflags + target_description.build.cppflags
    target.ldflags = project_descriptor.build.ldflags + target_description.build.ldflags
    target.libraries = project_descriptor.build.libraries + target_description.build.libraries
    target.include_directories.update(project_descriptor.build.includes)
    target.include_directories.update(project_descriptor.targets[target_name].build.includes)
    target.main = target_description.main
    target.image = target_description.image
    target.dockerfile = target_description.dockerfile
    target.test_image = target_description.test_image
    target.test_dockerfile = target_description.test_dockerfile
    target.toolchain_prefix = target_description.toolchain_prefix
    target.post_build = target_description.post_build
    compose_packages(project_descriptor.build.packages, target)
    compose_packages(target_description.build.packages, target)
    for bit_description in project_descriptor.build.bits.values():
        compose_bit(bit_description, target)
    for bit_description in target_description.build.bits.values():
        compose_bit(bit_description, target)
    return target


def compose_tests(target_name, project_descriptor, project):
    target_description = project_descriptor.targets.get(target_name, TargetDescription(target_name))
    project.test.cflags = project_descriptor.test.cflags + target_description.test.cflags
    project.test.cppflags = project_descriptor.test.cppflags + target_description.test.cppflags
    project.test.ldflags = project_descriptor.test.ldflags + target_description.test.ldflags
    project.test.libraries = project_descriptor.test.libraries + target_description.test.libraries
    project.test.include_directories.update(project_descriptor.test.includes)
    project.test.include_directories.update(target_description.test.includes)
    compose_packages(project_descriptor.test.packages, project.test)
    compose_packages(target_description.test.packages, project.test)

    for bit_description in project_descriptor.test.bits.values():
        compose_bit(bit_description, project.test)

    for test_file in filesystem.find('tests', 'test_*.cpp'):
        name = test_file.split('/')[-1].split('.')[0]
        test_suite = TestSuite(name, test_file)
        project.test.test_suites.append(test_suite)


def compose_packages(packages, target):
    for package_description in packages:
        package = Package(package_description.path)
        package.sources = filesystem.find(package.path, '*.cpp') + filesystem.find(package.path, '*.c')
        package.cflags = package_description.cflags + target.cflags
        package.cppflags = package_description.cppflags + target.cppflags
        target.packages.append(package)
        target.include_directories.add(package_include_directory(package_description))
        package.include_directories = target.include_directories


def compose_bit(bit_description, target):
    adjust_bit_packages_base_path(bit_description, bit_description.build.packages)
    add_packages_to_target_includes(bit_description.build.packages, target)
    bit_target_name = get_bit_target_name(bit_description)
    adjust_bit_packages_base_path(bit_description, bit_description.targets[bit_target_name].build.packages)
    add_packages_to_target_includes(bit_description.targets[bit_target_name].build.packages, target)
    bit_target = compose_target(bit_target_name, bit_description)
    target.bits.append(bit_target)
    add_cflags_to_bit_packages(bit_target, bit_description.declared_bit.cflags)
    add_cppflags_to_bit_packages(bit_target, bit_description.declared_bit.cppflags)


def get_bit_target_name(bit_description):
    if bit_description.declared_bit.target in bit_description.targets:
        return bit_description.declared_bit.target
    else:
        return DEFAULT_TARGET


def adjust_bit_packages_base_path(bit_description, packages):
    for package_description in packages:
        package_description.path = f'{bit_directory(bit_description.name, bit_description.version)}/{package_description.path}'


def add_packages_to_target_includes(packages, target):
    for package_description in packages:
        target.include_directories.add(package_include_directory(package_description))


def package_include_directory(package_description):
    return filesystem.parent_directory(package_description.path)


def add_cflags_to_bit_packages(bit_target, cflags):
    for package in bit_target.packages:
        package.cflags.extend(cflags)


def add_cppflags_to_bit_packages(bit_target, cppflags):
    for package in bit_target.packages:
        package.cppflags.extend(cppflags)


class TargetNotDescribed(RuntimeError):
    pass



from cpm.infrastructure import filesystem
from cpm.domain.constants import PROJECT_DESCRIPTOR_FILE


class ProjectPackager(object):
    def pack(self, project_descriptor, build_directory):
        if not project_descriptor.build_packages():
            raise PackagingFailure(cause='project contains no packages')

        if filesystem.directory_exists(build_directory):
            raise PackagingFailure(cause='build directory exists')

        filesystem.create_directory(build_directory)
        filesystem.copy_file(PROJECT_DESCRIPTOR_FILE, f'{build_directory}/project.yaml')
        for included_file in project_descriptor.included_files():
            filesystem.copy_file(PROJECT_DESCRIPTOR_FILE, f'{build_directory}/{included_file}')
        for package in project_descriptor.build_packages():
            filesystem.copy_directory(package.path, f'{build_directory}/{package.path}')
        filesystem.zip(build_directory, f'{project_descriptor.name}')
        filesystem.remove_directory(build_directory)

        return f'{project_descriptor.name}.zip'


class PackagingFailure(RuntimeError):
    def __init__(self, cause='packaging failure'):
        self.cause = cause
